# -*- coding: utf-8 -*-
require 'ec_rider/ec_helper'

module EcRider
  module EcHelper
    module OrderTag
      extend MethodDescription

      desc <<-EOS
        [概要：クレジットカードの認証エラーが発生した場合、その中身を実行します。]
        [引数：なし]
        [戻り値：なし]
        [使用例：
          <ec:if_cedit_auth_failed>
            <p>申し訳ございません。お手数ですが、以下のご入力内容をお確かめ下さい。</p>
            <ec:order_credit_auth_error>
          <ec:if_cedit_auth_failed>
        ]
      EOS
      def if_cedit_auth_failed(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
        if session[:credit_auth_error]
          yield
        end
      end

      desc <<-EOS
        [概要：クレジットカードの認証エラーを表示します]
        [引数：なし]
        [戻り値：なし]
        [使用例：ec:if_cedit_auth_failedを参照してください。]
      EOS
      def order_credit_auth_error
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
        return "" unless  auth_error = session[:credit_auth_error]
        error_msg = ""
        for error in auth_error
          error_msg << "#{error}<br/>"
        end
      end

      desc <<-EOS
        [概要:配送商品一覧を出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_product_list>
             <ec:order_product_name />
             <ec:order_product_price />
             <ec:order_product_qty />
             <ec:order_product_price_x_qty />
             <ec:order_product_discount_price />
             <ec:order_product_sub_total_price />
             <ec:order_product_total_price />
           </ec:order_product_list>
           }}}
        ]
      EOS
      def order_product_list(&block)
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].item_list_order_by_present.nil?
        session[:cart].item_list_order_by_present.each do |order_product|
          @order_product = order_product
          begin
            @template.concat(@template.capture(&block))
          ensure
            @order_product = nil
          end
        end
      end
      
      desc <<-EOS
        [概要:配送商品の種類数を出力します。]
        [引数:なし]
        [使用例:
           {{{
           <ec:order_product_count>
           }}}
        ]
      EOS
      def order_product_count
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].product_list.nil?
        session[:cart].product_list.length
      end

      desc <<-EOS
        [概要:配送商品の画像を表示します。配送商品一覧内で使用してください。]
        [引数 type(任意):商品画像のサイズを指定します。(large →320x320サイズの画像を表示します。, medium→140x140サイズの画像を表示します。, small→46x46サイズの画像を表示します。)]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_image(*args, &block)
        raise UsageError, "order_product_nameはec:order_product_listの中で使用してください" unless @order_product
        type, options = extract(args, :type => nil)
        image = @order_product.product_sku.product.main_image
        options["alt"] ||= @order_product.name
        @template.image_tag(image.public_filename(type), options) unless image.nil?
      end

      desc <<-EOS
        [概要:配送商品の名前を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_name(*args, &block)
        raise UsageError, "order_product_nameはec:order_product_listの中で使用してください" unless @order_product
        @order_product.name if @order_product.name
      end

      desc <<-EOS
        [概要:配送商品の獲得ポイントを表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_given_point(*args, &block)
        @template.number_with_delimiter(@order_product.given_point) if @order_product.given_point
      end

      desc <<-EOS
        [概要:配送商品の会員ランク毎の単価を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_price(*args, &block)
        @template.number_with_delimiter(@order_product.unit_price) if @order_product.unit_price
      end
      alias_method :order_product_normal_price, :order_product_price


      desc <<-EOS
        [概要:配送商品の数量を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_qty(*args, &block)
        @template.number_with_delimiter(@order_product.qty) if @order_product.qty
      end

      desc <<-EOS
        [概要:配送商品の割引金額を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_discount_price(options = {}, *args, &block)
        format = options[:format] || ''
        value = @order_product.discount_amount
        if value.to_i > 0
          return @template.number_with_delimiter(value) + format
        else
          return "0"
        end
      end

      desc <<-EOS
        [概要:配送商品の小計(割引金額適用前)を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_price_x_qty(*args, &block)
        @template.number_with_delimiter(@order_product.subtotal_amount_for_discount) if @order_product.subtotal_amount_for_discount
      end

      desc <<-EOS
        [概要:配送商品の小計(割引金額適用後)を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_sub_total_price(*args, &block)
        @template.number_with_delimiter(@order_product.total_amount) if @order_product.total_amount
      end

      desc <<-EOS
        [概要:注文商品の割引適用後の合計金額を表示します。ec:order_confirmation_form内の配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：<ec:order_product_total_price />]
      EOS
      def order_product_total_price(*args, &block)
        @template.number_with_delimiter(session[:cart].total_amount)
      end

      desc <<-EOS
        [概要:注文商品の合計から求められる割引金額を表示します。(商品単品の割引金額は加算されません。)ec:order_confirmation_form内の配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例:<ec:order_product_total_price />]
      EOS
      def order_product_total_discount_price(*args, &block)
        @template.number_with_delimiter(session[:cart].discount_amount)
      end

      desc <<-EOS
        [概要:注文合計獲得ポイントを表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_total_given_point]
      EOS
      def order_total_given_point(*args, &block)
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].item_list.nil?
        @template.number_with_delimiter(session[:cart].total_given_point)
      end

      desc <<-EOS
        [概要:注文後利用可能ポイントを表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_total_point]
      EOS
      def order_total_point(*args, &block)
        raise UsageError, "カートに商品がありません" if session[:cart].nil? || session[:cart].item_list.nil?
        point = 0
        point = (point + UserPoint.total_point_for_user(@template.current_user.id).to_i) if @template.current_user
        point = (point + session[:cart].total_given_point.to_i)
        @template.number_with_delimiter(point)
      end

      desc <<-EOS
        [概要:配送料の合計を表示します。ec:order_confirmation_form内の配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_shipping_cost(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @order
        # 2009/11/25 現在、orderとorder_shippingは1対1なので、
        # order_shippingsメソッドで返却される配列の一番目のオブジェクト取得しています。
        order_shipping = @order.object.order_shippings.first
        shipping_cost = order_shipping.shipping_cost
        # shipping_costは代引き手数料込みの金額なので、代引き手数料を引いた金額を取得しています。
        shipping_cost = shipping_cost - order_shipping.cod_charge if order_shipping.cod_charge
        @template.number_with_delimiter(shipping_cost)
      end
      desc <<-EOS
        [概要:配送商品の合計を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：ec:order_product_listを参照してください。]
      EOS
      def order_product_total_price_include_shipping_cost(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @order
        order_shipping = @order.object.order_shippings.first
        @template.number_with_delimiter(order_shipping.total_amount + order_shipping.shipping_cost)
      end

      desc <<-EOS
        [概要:代引き手数料を表示します。ec:order_confirmation_form内の配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：{{{代金引換手数料：<ec:order_product_cod_charge />円}}}→代金引換手数料：1,000円]
      EOS
      def order_product_cod_charge(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @order
        order_shipping = @order.object.order_shippings.first
        @template.number_with_delimiter(order_shipping.cod_charge)
      end

      desc <<-EOS
        [概要:支払い方法が代引き手数料の場合中身を表示します。配送商品一覧内で使用してください。]
        [引数:なし]
        [使用例：{{{
                 <ec:if_order_product_cod_charge>
                     代金引換手数料：<ec:order_product_cod_charge />円
                 </ec:if_order_product_cod_charge>
                  }}} → 支払方法が代金引き当ての場合、代金引換手数料を表示します。
        ]
      EOS
      def if_order_product_cod_charge(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @order
        order_shipping = @order.object.order_shippings.first
        if order_shipping.payment_type_div == OrderShipping.payment_type_div_id_by_key(:cod)
          yield if block_given?
        end
      end

      desc <<-EOS
        [概要:お届け先指定・お支払方法用の確認画面へ進むボタンを出力します。お届け先指定・お支払方法画面内で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:button_to_confirmation_order>
             次へ進む→
           </ec:button_to_confirmation_order>
           }}}
        ]
      EOS
      def button_to_confirmation_order(*args, &block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@order_form)
        button_name = @template.capture(&block) if block_given?
        result = @template.submit_tag(button_name, {:id => "send"})
        @template.concat(result) if result && block_given?
      end
      
      desc <<-EOS
        [概要:Alipay決済画面へ遷移するボタンを出力します。Alipay決済情報の入力画面で使用してください。]
        [引数:なし]
        [使用例:
           {{{
           <ec:button_to_sps_payment>
             処理を続ける→
           </ec:button_to_sps_payment>
           }}}
        ]
      EOS
      def button_to_sps_payment(*args, &block)
        sps_paramter_history = @template.instance_variable_get(:@sps_parameter_history)
        raise UsageError, "ここでは使用できません。" unless sps_paramter_history
        
        attribute_keys_for_post = [
          'pay_method', 'merchant_id', 'service_id', 'cust_code', 'sps_cust_no', 'sps_payment_no', 'sps_order_id', 'item_id',
          'pay_item_id', 'item_name', 'tax', 'amount', 'pay_type', 'auto_charge_type', 'service_type', 'div_settele',
          'last_charge_month', 'camp_type', 'tracking_id', 'terminal_type', 'success_url', 'cancel_url', 'error_url',
          'pagecon_url', 'free1', 'free2', 'free3', 'free_csv', 'request_date', 'limit_second', 'sps_hashcode' ]
        
        key_names = {'sps_order_id' => 'order_id'}
        
        @template.form_tag(EcRider.config.sps_uri) do
          attribute_keys_for_post.each do |key|
            name = key_names[key] ? key_names[key] : key
            @template.concat(@template.hidden_field_tag(name, sps_paramter_history.attributes[key].to_s))
          end
          
          result = @template.submit_tag(@template.capture(&block), {:id => "send"}) if block_given?
          @template.concat(result) if result && block_given?
        end
      end

    end
  end
end
