# -*- coding: utf-8 -*-
class Campaign < ActiveRecord::Base
  include SafeEval

  # その時点で有効なキャンペーン
  # priorityの大きい順にソートします。
  named_scope :availables, lambda {|*args| 
    t = args.first || Time.now
    {
      :conditions => ["started_at <= ? AND finished_at >= ?", t, t],
      :order => "priority desc, id asc"
    }
  }

  # キャンペーン条件
  has_many :conditions, :class_name => 'CampaignCondition', :foreign_key => 'campaign_id'
  accepts_nested_attributes_for :conditions, :allow_destroy => true

  # キャンペーン対象商品
  has_many :targets, :class_name => 'CampaignTargetProductSku', :foreign_key => 'campaign_id'
  has_many :target_skus, :through => :targets, :source => :product_sku
  accepts_nested_attributes_for :targets, :allow_destroy => true
  attr_accessor :targets_modified


  # キャンペーン効果
  has_many :effects, :class_name => 'CampaignEffect', :foreign_key => 'campaign_id'
  accepts_nested_attributes_for :effects, :allow_destroy => true

  def effect_type_key
    effect = effects.first # １キャンペーン１効果という仕様です
    effect ? effect.type_key : nil
  end


  validates_presence_of :code
  validates_presence_of :name
  validates_presence_of :started_at, :finished_at
  validates_presence_of :priority
  validates_inclusion_of :priority, :in => 1..999
  validates_numericality_of :priority

  before_validation :set_conditions_no

  def set_conditions_no
    conditions.each_with_index do |condition, index|
      condition.no = index
    end
  end

  def validate
    super
    validate_priority_duplication_in_same_period
    validate_brace_count
  end

  private
  
  def validate_brace_count
    count = 0
    conditions.each do |condition|
      count += condition.brace_begin_count
      count -= condition.brace_end_count
      if count < 0
        errors.add_to_base("条件の括弧の数が合っていません")
        return false
      end
    end
    if count != 0
      errors.add_to_base("条件の括弧の数が合っていません")
      return false
    end
    return true
  end

  def validate_priority_duplication_in_same_period
    where, params = [], []
    where << "(started_at <= ? AND finished_at >= ?)"; params.concat([self.started_at, self.finished_at])
    where << "(started_at >= ? AND finished_at <= ?)"; params.concat([self.started_at, self.finished_at])
    where << "(started_at <= ? AND finished_at > ?)"; params.concat([self.started_at, self.started_at])
    where << "(started_at < ? AND finished_at >= ?)"; params.concat([self.finished_at, self.finished_at])
    where = ['(%s)' % where.join(" OR ")]
    where << "priority = ?" ; params << self.priority
    unless new_record?
      where << "id <> ?" ; params << self.id
    end
    if Campaign.count(:conditions => [where.join(" AND ")].concat(params)) > 0
      errors.add_to_base("期間が重複する場合、優先度は同じ値を設定することができません。")
    end
  end



  public

  def mark_targets_for_destruction(modified = targets_modified)
    if ActiveRecord::ConnectionAdapters::Column.value_to_boolean(modified)
      self.targets.each do |target|
        target.mark_for_destruction
      end
    end
  end


  def build_conditions(options = nil)
    options = {
      :numbering_conditions => true
    }.update(options || {})
    numbering_conditions if options[:numbering_conditions]
    conditions.map{|condition| condition.build}.join(' ')
  end

  def numbering_conditions
    conditions.each_with_index do |condition, index|
      condition.no = index
    end
  end

  # カートがキャンペーンの対象となるかどうかを判断します。
  def match?(cart)
    match_to_target?(cart) && match_to_conditions?(cart.to_campaign_context)
  end

  # キャンペーンの対象商品SKUをカートが持っているかどうかを判断します。
  # targetsが空かtargetsに該当し、かつ条件に合致する
  def match_to_target?(cart)
    return true if targets.empty?
    contained_ids = self.target_sku_ids & cart.items.map{|(key, item)| item.product_sku_id}
    !contained_ids.empty?
  end

  # 与えられたコンテキストでこの条件を評価します
  # 余計なアクセスなどを行わないように、スレッドの$SAFE(セーフレベル)を指定して、安全に実行できるようにしています。
  # スレッドは平行処理するためではないので、このメソッドはスレッドを実行して終了を待って戻り値を返します。
  def match_to_conditions?(context = {})
    return true if conditions.empty?
    script = build_conditions.untaint # 汚染マークを外さないと下のinstance_evalでSecurityErrorが発生する
    !!safe_instance_eval(script, context, 3)
  end

  # 渡されたカートがこのキャンペーンの適用対象ならば適用します。
  def apply_if_match(cart)
    apply(cart) if match?(cart)
  end
  
  # 渡されたカートがこのキャンペーンを適用します。
  def apply(cart)
    effects.each do |effect|
      effect.apply(cart)
    end
  end

  # 対象がカートの場合にtrueを返します
  # このメソッド名の"targets"は名詞の複数形ではなく、動詞の三人称単数形です。
  def targets_cart?
    targets.empty?
  end

  # 対象がカートの明細の場合にtrueを返します。
  # このメソッド名の"targets"は名詞の複数形ではなく、動詞の三人称単数形です。
  def targets_items?
    !targets_cart?
  end

  # 効果にプレゼントを含むキャンペーンで、プレゼント品が在庫切れならtrueを返します。
  def present_but_no_stock?(cart = nil)
    effects.any?{|effect| effect.present_but_no_stock?(cart)}
  end


  #
  # = キャンペーン一覧検索入力フォーム(管理画面用) =
  #
  # 作りかけです。　OSS対応までpendingになったため
  #
  class SearchConditionForm < EcRider::ConditionForm
    attr_accessor :code, :name, :campaign_started_at, :campaign_finished_at, :page_name

    def to_find_options(options ={})
      joins = []
      where = Condition.block do |c|
        # キャンペーンコードの条件
        c.and("code", "#{self.code}") unless self.code.blank?
        # キャンペーン名称の条件
        c.and("name", 'like', "#{self.name}") unless self.name.blank?
        # ページ名称の条件
        c.and("cms_pages.action", "#{self.page_name}") unless self.page_name.blank?
        # キャンペーン期間の条件
#        c.and("locale", "#{self.locale}") unless self.name.blank?
      end
      result = {:conditions => where}
      result[:joins] = joins.uniq.join(" ")
      result.update(options) if options
      return result
    end
  end
end
