# -*- coding: utf-8 -*-
class CampaignCondition < ActiveRecord::Base
  include SafeEval

  belongs_to :campaign

  # 削除を指示するためのフラグ
  attr_accessor :deleting

  validates_presence_of :no
  validates_presence_of :type_cd
  validates_presence_of :operator_cd
  # validates_presence_of :value # valueはnull可
  validates_presence_of :connector_cd
  validates_presence_of :brace_begin_count, :brace_end_count


  before_validation :set_brace_counts

  def set_brace_counts
    self.connector_key = :and if connector_key.nil?
    self.brace_begin_count = 0 if brace_begin_count.blank?
    self.brace_end_count ||= 0 if brace_end_count.blank?
  end

  # 接続演算子
  # 論理演算子については、以下を参照
  #   http://ufcpp.net/study/computer/logic.html
  selectable_attr :connector_cd do
    entry '01', :and , 'かつ'  , :operator => '&&'
    entry '02', :or  , 'または', :operator => '||'
  end

  # 演算子
  # 比較演算子については、以下を参照してください
  #   http://datamapper.rubyforge.org/dm-core/
  #   の Symbol
  selectable_attr :operator_cd do
    entry '01', :lt  , 'より小さい', :ruby => "<"
    entry '02', :gt  , 'より大きい', :ruby => ">"
    entry '03', :lte , '以下'     , :ruby => "<="
    entry '04', :gte , '以上'     , :ruby => ">="
    entry '05', :eql , '等しい'   , :ruby => "=="
    entry '06', :not , '以外'     , :ruby => "!="
    entry '07', :like, 'を含む'   , :ruby => "=~"
    entry '08', :not_like, 'を含まない'   , :ruby => "!~"
  end

  # 項目種
  selectable_attr :type_cd do
    entry '01', :user_gender, '性別', :accessor => 'user.sex_key.to_s', :func => "'%s'", :availables => [:eql, :not]

    string_func = Proc.new do |entry, condition, value|
      case condition.operator_key
      when :like, :not_like then
        "Regexp.new(Regexp.escape('#{value.to_s}'))"
      else
        "'#{value.to_s}'"
      end
    end

    with_options(:availables => [:eql, :not, :like], :func => string_func) do |s|
      s.entry '02', :user_country, '国', :accessor => 'user.address_country_code'
      s.entry '03', :user_address, '住所', :accessor => "user.address_for_campaign_condition"
    end
    entry '04', :user_address_isolated, '離島', :availables => [:eql], :accessor => "user.address_isolated?", :func => "false"

    entry '05', :user_age, '年齢', :accessor => 'user.age', :func => "CampaignCondition.parse_age('%s')"

    with_options(:availables => CampaignCondition.operator_keys - [:like]) do |s|
      s.entry '07', :user_birthday        , '誕生日(年月日)', :accessor => 'user.birthday'         , :func => "CampaignCondition.parse_date('%s')"
      s.entry '08', :user_birthday_of_year, '誕生日(月日)'  , :accessor => 'user.birthday_of_year' , :func => "CampaignCondition.parse_date_of_year('%s')"
      s.entry '09', :user_birth_year      , '誕生日(年)'    , :accessor => 'user.birth_year.to_i'  , :func => "CampaignCondition.parse_year('%s')"
      s.entry '10', :user_birth_month     , '誕生日(月)'    , :accessor => 'user.birth_month.to_i' , :func => "CampaignCondition.parse_month('%s')"
      s.entry '11', :user_birth_day       , '誕生日(日)'    , :accessor => 'user.birth_day.to_i'   , :func => "CampaignCondition.parse_day_of_month('%s')"
    end

    with_options(:availables => CampaignCondition.operator_keys - [:like]) do |s|
      s.entry '12', :now_date        , '現時刻(年月日)', :accessor => 'Date.today'                  , :func => "CampaignCondition.parse_date('%s')"
      s.entry '13', :now_date_of_year, '現時刻(月日)'  , :accessor => "Date.today.strftime('%m/%d')", :func => "CampaignCondition.parse_date_of_year('%s')"
      s.entry '14', :now_year        , '現時刻(年)'    , :accessor => 'Date.today.year.to_i'        , :func => "CampaignCondition.parse_year('%s')"
      s.entry '15', :now_month       , '現時刻(月)'    , :accessor => 'Date.today.month.to_i'       , :func => "CampaignCondition.parse_month('%s')"
      s.entry '16', :now_day         , '現時刻(日)'    , :accessor => 'Date.today.day.to_i'         , :func => "CampaignCondition.parse_day_of_month('%s')"
      s.entry '17', :now_day_of_week , '現時刻(曜日)'  , :accessor => 'Date.today.cwday.to_i'       , :func => "CampaignCondition.parse_day_of_week('%s')"
      s.entry '18', :now_time        , '現時刻(時分)'  , :accessor => "Time.now.strftime('%H:%M')"  , :func => "CampaignCondition.parse_time_of_day('%s')"
    end

    with_options(:availables => CampaignCondition.operator_keys - [:like], :func => "'%s'.to_i") do |s|
      s.entry '19', :user_rank, '会員ランク', :accessor => 'user.user_rank'
      s.entry '20', :user_total_point, '保有ポイント', :accessor => 'user.total_point.to_i'
      s.entry '21', :cart_product_subtotal_price, '注文小計額', :accessor => 'cart.subtotal_amount.to_i'
      s.entry '23', :cart_product_total_price, '注文合計額', :accessor => 'cart.total_amount.to_i'
    end
    entry '22', :member_category_div, '会員種別', :availables => [:eql], :accessor => 'user.member_category_div', :func => string_func
  end

  # Campaignで結合してしようするための接続演算子付きのスクリプトを生成します。
  def build
    result = ''
    if no > 0
      result << connector_entry[:operator] << " "
    end
    result << "(" * (brace_begin_count || 0)
    result << build_script
    result << ")" * (brace_end_count || 0)
    result
  end


  # 評価用のRubyスクリプトを生成します
  def build_script
    errors.clear
    unless validate_to_build_script
      raise ActiveRecord::RecordInvalid.new(self)
    end
    func = type_entry[:func]
    value_script = func.respond_to?(:call) ? func.call(type_entry, self, value) : func.to_s % value
    "%s %s %s" % [
      type_entry[:accessor], operator_entry[:ruby], value_script
    ]
  end

  # build_scriptを実行可能かどうかを判断し、できない場合はerrorsにメッセージを追加します。
  def validate_to_build_script
    errors.add_on_blank(:type_cd)
    errors.add_on_blank(:operator_cd)
    errors.empty?
  end


  # 与えられたコンテキストでこの条件を評価します
  # 余計なアクセスなどを行わないように、スレッドの$SAFE(セーフレベル)を指定して、安全に実行できるようにしています。
  # スレッドは平行処理するためではないので、このメソッドはスレッドを実行して終了を待って戻り値を返します。
  def match?(context = {})
    !!safe_instance_eval(build_script, context, 3)
  end

  def self.parse_date(str)
    (str =~ /today/i) ? Date.today : Date.parse(str)
  end

  def self.parse_date_of_year(str)
    # 2000年は閏年
    d = (str =~ /today/i) ? Date.today : Date.new(2000, *str.split('/')[-2..-1].map(&:to_i))
    d.strftime('%m/%d')
  end

  def self.parse_year(str)
    (str =~ /today/i) ? Date.today.year : str.to_i
  end

  def self.parse_month(str)
    (str =~ /today/i) ? Date.today.month : str.to_i
  end

  def self.parse_day_of_month(str)
    (str =~ /today/i) ? Date.today.day : str.to_i
  end

  week_days = {
    1 => %w(月 月曜日 Mon Monday),
    2 => %w(火 火曜日 Tue Tuesday),
    3 => %w(水 水曜日 Wed Wednesday),
    4 => %w(木 木曜日 Thu Thursday),
    5 => %w(金 金曜日 Fri Friday),
    6 => %w(土 土曜日 Sat Saturday),
    7 => %w(日 日曜日 Sun Sunday)
  }
  WEEK_DAYS = {}
  week_days.each do |num, keys|
    keys.each{|key| WEEK_DAYS[key] = num}
  end
  WEEK_DAYS.freeze

  def self.parse_day_of_week(str)
    WEEK_DAYS[str]
  end

  def self.parse_time_of_day(str)
    str = str.split(/\s/).last
    raise ArgumentError, "Invalid time format #{str}" unless str =~ /\d{1,2}:\d{1,2}/
    d = (str =~ /today/i) ? Time.now : Time.local(2000, 1, 1, *str.split(':')[0..1].map(&:to_i))
    d.strftime('%H:%M')
  end

  def self.parse_age(str)
    num_str = (str || '').gsub(/\D/, '')
    raise ArgumentError, "Invalid age #{str}" if num_str.blank?
    num_str.to_i
  end

  # 項目種別から選択可能な演算子を取得する
  # 管理画面の編集画面セレクトボックスに使用しています
  def self.operator_cd_values(type_code)
    cam = CampaignCondition.new(:type_cd => type_code)
    availables = cam.type_entry[:availables]
    result = []
    availables.each do |available_key|
      result << [CampaignCondition.operator_name_by_key(available_key), CampaignCondition.operator_id_by_key(available_key)]
    end
    return result
  end

  # 項目種別から表示する区分値名を表示する
  # 例) 項目が｢会員種別｣の場合項目名称区分から｢会員種別名称｣を取得して表示
  def value_for_type
    case self.type_key
    when :member_category_div
      kbn_code = KbnCode.find(:first, :select =>'kbn_code_names.name as name',
        :conditions => ['kbn = ? AND code = ? AND kbn_code_names.locale = ? ', KbnCode.name_div_id_by_key(:member_category), self.value, 'ja'],
        :joins => "LEFT OUTER JOIN kbn_code_names on kbn_codes.id = kbn_code_names.kbn_code_id")
      return kbn_code.name
    end
    return self.value
  end
end
