# -*- coding: utf-8 -*-
#
# Order
#  amount（メソッド）  | 注文の請求金額(order_shipping.total_amount + order_shipping.shipping_cost)
#  -                   | 割引前の合計金額(sum(order_item.subtotal_amount)) = 輸出用invoiceに記載する総額
# 
# OrderShipping
#  total_amount        | 売上金額 = 送料、代引き手数料を含まない注文金額(subtotal_amount - discount_amount)
#  subtotal_amount     | 注文明細の合計（sum(order_item.total_amount))
#  discount_amount     | 注文に対する割引金額
#  shipping_cost       | 送料＋代引き手数料
#  cod_charge          | 代引き手数料
#  -                   | 送料 = order_shipping.shipping_cost - order_shipping.cod_charge
#
# OrderItem
#  unit_price          | 商品単価
#  qty                 | 数量
#  shipping_cost       | 送料（STEP2では使わない）
#  subtotal_amount     | 割引前の合計金額(order_item.unit_price * order_item.qty) = 輸出用invoiceに記載する商品明細の合計金額
#  discount_amount     | 割引金額
#  total_amount        | 割引適用後の合計金額（order_item.subtotal_amount - order_item.discount_amount)
#
require 'digest/sha1'

class Order < ActiveRecord::Base
  versioned

  include EcRider::I18nSupport

  acts_as_paranoid

  has_many   :order_shippings, :class_name => 'OrderShipping', :foreign_key => 'order_id', :dependent => :destroy
  has_many   :gmo_transaction_histories, :class_name => '::GmoTransactionHistory', :foreign_key => 'order_id'  # 決済履歴

  belongs_to :user
  belongs_to :gmo_credit_card, :class_name => 'GmoCreditCard', :foreign_key => 'gmo_credit_card_id'

  belongs_to :include_order, :class_name => 'Order', :foreign_key => 'include_order_id'
  has_many   :included_orders, :class_name => 'Order', :foreign_key => 'include_order_id'

  has_many  :mail_deliveries, :class_name => 'MailDelivery', :foreign_key => 'order_id'

  has_many    :user_points, :class_name => 'UserPoint', :foreign_key => 'order_id'
  has_many    :accountings, :class_name => 'Accounting', :foreign_key => 'order_id'

  # 注文コードの採番
  after_create :generate_ec_order_code

  before_validation :set_test_flg

  def set_test_flg
    self.test_flg = true if self.test_flg.nil? && EcRider.config.ec_test
  end

  # クレジット実行区分
  selectable_attr :on_line_payment_exe_div do
    entry 1, :not_need, 'オンライン決済不要'
    entry 2, :not_exe, '未実施'
    entry 3, :error, 'オーソリエラー'
    entry 4, :success, 'オーソリ完了'
  end

  # 購入サイト区分
  selectable_attr :site_type_div do
    entry '1', :pc, 'PC'
    entry '2', :mobile, 'モバイル'
  end

  # 引数に渡された検索条件で検索します
  def self.find_any_condition(condition, limit = nil, offset = nil)
    if condition.is_a?(Hash)
      condition = condition.reject{|key, val| val.empty?}
      condition = SearchInfo.new(condition)
    end
    find(:all, :include => SearchInfo.create_include, :conditions => condition.create_condition,
      :limit => limit, :offset => offset, :order => 'orders.id DESC')
  end

  # 渡された検索条件にHITする注文の合計金額を返す
  def self.total_amount(condition)
    condition = condition.reject{|key, val| val.empty?}
    condition = SearchInfo.new(condition) if condition.is_a?(Hash)
    # TODO 計算した結果を取得するように変更する
    order_shippings = find(:all,
      :select => 'order_shippings.total_amount',
      :joins => SearchInfo.create_join,
      :conditions => condition.create_condition,
      :group => 'order_shippings.id')
    order_shippings.sum{|shipping| shipping.total_amount.to_i}
  end

  # 渡された検索条件にHITする注文の合計送料を返す
  def self.total_shipping_cost(condition)
    condition = condition.reject{|key, val| val.empty?}
    condition = SearchInfo.new(condition) if condition.is_a?(Hash)
    # TODO 計算した結果を取得するように変更する
    order_shippings = find(:all,
      :select => 'order_shippings.shipping_cost',
      :joins => SearchInfo.create_join,
      :conditions => condition.create_condition,
      :group => 'order_shippings.id')
    order_shippings.inject(0){|sum, chapter| sum + chapter.shipping_cost.to_i}
  end

  #
  # 支払い方法がクレジットの場合に、OrderShippingを配列に入れて返す
  #
  def credit_order_shippings
    order_shipping = self.order_shippings.first #1注文に配送先は一つ
    order_shipping.payment_type_div_key == :credit ? [order_shipping] : []
  end

  #
  # = クレジット決済処理実行 =
  #
  #　事後条件：クレジット支払の履歴オブジェクトを返す。クレジット決済の注文見出しが含まれない場合nilを返す。
  def credit_card_payment(credit_card)
    # TODO
    # GMO決済もSPS決済と同じメソッド呼び出して実行できるようにしたいです。
    # そのタイミングでここの分岐条件は削除します。
    case EcRider.config.payment_system_div
    when CreditCardPayment.payment_system_div_id_by_key(:gmo)
      request_gmo_credit_card_payment(credit_card)
    when CreditCardPayment.payment_system_div_id_by_key(:sps)
      # SPS決済
      payment = CreditCardPayment.new(self, credit_card)
      payment.process
    else
      raise ArgumentError, "指定された決済システムサービスは、ご利用できません。"
    end
  end

  # GMO決済
  def request_gmo_credit_card_payment(credit_card)
    #在庫数チェック
    self.check_stocks
    credit_history, member_id, card_seq = self.credit_each_settlement(credit_card.card_no, credit_card.expire, credit_card.card_name, nil) if credit_card
    if credit_history
      # クレジット処理結果がエラーの場合
      if credit_history.err_code
        error = CreditAuthError.new(i18n_message(credit_history.is_system_err_code? ? :EM990004 : :EM020010))
        error.error_payment = credit_history
        raise error
      end
      # オンライン決済実行区分を成功に設定
      self.on_line_payment_exe_div_key = :success
      if member_id
        # 決済ユーザIDをGMOに登録した場合は、決済ユーザIDを保存する
        gmo_credit_card = GmoCreditCard.new(:credit_user_code => member_id, :user_id => get_user.id, :card_seq => card_seq)
        self.gmo_credit_card = gmo_credit_card
      end
      credit_history.save!
    end
  end

  # 注文の請求金額(order_shipping.total_amount + order_shipping.shipping_cost)
  def amount
    total_amount = 0
    shipping_cost = 0
    self.order_shippings.each do |shipping|
      total_amount  += shipping.total_amount
      shipping_cost += shipping.shipping_cost
    end
    total_amount + shipping_cost
  end

  def sps_item_id
    # 注文商品から１つ選択し、その商品SKUのIDを設定しています。
    self.order_shippings[0].order_items[0].product_sku_id
  end

  # クレジット処理例外
  # 決済システムの処理結果がNGの際に発生する
  class CreditAuthError < StandardError
    attr_accessor :error_payment # 例外を起こした決済オブジェクト。create_payment_error_historyメソッドで例外の記録をDBにinsertします
  end

  # GMOクレジット決済処理実行
  # オンライン決済履歴のインスタンスを返します。
  def credit_each_settlement(card_no, expire, card_name, card_kind = nil)
    if credit_order_shippings.size > 0
      total_amount = 0;
      shipping_cost = 0;

      credit_order_shippings.each do |shipping|
        total_amount  += shipping.total_amount
        shipping_cost += shipping.shipping_cost
      end

      member_id = nil
      # クレジットカードのオーソリ
      if has_regurar_order?
        # 決済用のユーザIDをGMOに登録する場合
        member_id = GmoMemberIdSeq.create_member_id
        payment_history, card_seq = CreditCardRegister.regular_settlement(total_amount, shipping_cost, card_no, expire, member_id, card_name, card_kind)
      else
        # 都度購入の場合
        payment_history = CreditCardRegister.each_settlement(total_amount, shipping_cost, card_no, expire, card_kind)
      end

      self.gmo_transaction_histories << payment_history
      return [payment_history, member_id, card_seq]    
    end
    return nil
  end

  # 定期購入商品が含まれる場合はtrueを返す
  def has_regurar_order?
    self.order_shippings.each do |order_shipping|
      order_shipping.order_items.each do |order_item|
        return true if order_item.course_div_key == :periodic
      end
    end
    return false
  end

  #
  # = キャンセル可能かどうかを判断する =
  #
  # :possible（キャンセル可能）
  # :impossible（キャンセル不可）
  # :cancel_request（キャンセル依頼中）
  # :canceled（キャンセル完了）
  #
  def cancel_status
    canceled = false
    cancel_request = false
    impossible = false
    # 複数お届け先のうち、すべてが未発送、または保留ならばキャンセル可能
    self.order_shippings.each do |order_shipping|
      if order_shipping.status_div_key == :canceled || order_shipping.status_div_key == :cancel_request ||
          order_shipping.status_div_key == :delivered || order_shipping.status_div_key == :deliver_request ||
          order_shipping.status_div_key == :deliver_advised
        impossible = true
      end
    end
    return :canceled if canceled
    return :cancel_request if cancel_request
    return :impossible if impossible
    return :possible
  end

  #
  # 値をセットしたOrderインスタンスを返す
  #
  def self.create_new_order(user, user_address, order_form, cart, site_type_div = :pc)
    order = Order.new(
      :ordered_at => Time.now,
      :site_type_div_key => site_type_div,
      :free_request => order_form.free_request,
      :locale => I18n.locale.to_s,
      :gift_flg => order_form.gift_flg,
      :send_invoice_separately_flg => order_form.send_invoice_separately_flg,
      :write_sender_flg => order_form.write_sender_flg,
      :include_order_id => order_form.include_order_id
    )
    # OrderShippingインスタンスを生成する
    unless cart.empty?
      order.order_shippings << create_new_order_shipping(cart, user, user_address, order_form)
    end
    return order
  end

  #
  # 値をセットしたOrderShippingのインスタンスを返す
  #
  def self.create_new_order_shipping(cart, user, user_address, order_form)
    shipping_cost = ShippingCost.calc_shipping_cost(
      user_address,
      cart.total_shipping_cost_point,
      cart.free_shipping_cost?)
    cod_charge = 0
    if order_form.normal_payment_type == OrderShipping.payment_type_div_id_by_key(:cod)
      #支払い方法が代引きの場合、手数料を加算する
      cod_charge = (ShopSetting.get_setting(:cod_charge, I18n.locale) || 0).to_i
      shipping_cost = shipping_cost + cod_charge
    end
    order_shipping = OrderShipping.new({
        :total_amount    => cart.total_amount, # 割引適用後の合計金額
        :subtotal_amount => cart.subtotal_amount, # 割引適用前の合計金額(明細の総合計)
        :discount_amount => cart.discount_amount, # 割引額
        :shipping_cost => shipping_cost,
        :payment_type_div => order_form.normal_payment_type,
        :status_div_key => :before_settlement,
        :csv_output_div_key => :not_yet,
        :cod_charge => cod_charge,
        :arrival_date => order_form.delivery_date,
        :arrival_hour => order_form.delivery_time
      })
    order_shipping.merge_only_need(user_address.attributes)
    order_shipping.merge_only_need(user.user_email.attributes)

    # OrderItemインスタンスを生成する
    order_shipping.order_items = cart.item_list.map do |cart_item|
      OrderItem.new(
        :product_name => cart_item.name,
        :order_product_name_1 => cart_item.product_sku.product_name_for_admin_ja,
        :order_product_name_2 => cart_item.product_sku.product.order_product_name_2,
        :course_div => cart_item.course_div,
        :unit_price => cart_item.unit_price,
        :qty => cart_item.qty,
        :subtotal_amount => cart_item.subtotal_amount,
        :discount_amount => cart_item.discount_amount,
        :total_amount    => cart_item.total_amount,
        :shipping_cost => 0,
        :product_sku_id => cart_item.product_sku.id
      )
    end
    return order_shipping
  end

  #
  # GMOに登録済みのクレジットカード情報を更新する
  # 
  def update_gmo_credit_card(card_no, expire)
    _gmo_credit_card = self.gmo_credit_card
    member_id = _gmo_credit_card.credit_user_code
    seq = _gmo_credit_card.card_seq
    # 料金と送料を取得
    total_amount = self.credit_order_shippings.sum(&:total_amount)
    shipping_cost = self.credit_order_shippings.sum(&:shipping_cost)
    return CreditCardRegister.update_card_info(member_id, card_no, expire, seq, total_amount, shipping_cost)
  end

  #
  # 注文コードを生成
  #
  def generate_ec_order_code
    self.ec_order_code = self.id.to_s.rjust(11, '0')
    self.save!
  end

  #
  # =注文情報詳細をテキストでフォーマットして返します=
  #
  def print_order_detail(type = :text, options = {})
    result = []
    total_info = self.total_info

    case self.locale
    when 'ja'
      print_locale = 'ja'
    when 'zh_CN'
      print_locale = 'zh_CN'
    else
      print_locale = 'en'
    end
    # 店舗宛出荷完了メールは日本語固定のためロケールを日本語に上書きします。
    unless options[:shipment_complete_mail].nil?
      print_locale = 'ja'
    end

    i18n_tmp_block(print_locale) do
      # http://api.rubyonrails.org/classes/I18n.html#M002268
      #
      # Scope can be either a single key, a dot-separated key or an array of keys or dot-separated keys.
      # Keys and scopes can be combined freely. So these examples will all look up the same short date format:
      # I18n.t 'date.formats.short'
      # I18n.t 'formats.short', :scope => 'date'
      # I18n.t 'short', :scope => 'date.formats'
      # I18n.t 'short', :scope => %w(date formats)
      #
      # TODO ここどっちだろう？ orderitems.print_order_detail かも
      I18n.with_options(:scope => 'orders.print_order_detail') do |i18n|
        yen = i18n.t('yen', :default => '')
        pre_name = i18n.t('pre_name', :default => '')
        pre_name = pre_name.blank? ? '' : (pre_name + ' ')
        post_name = i18n.t('post_name', :default => '')
        post_name = post_name.blank? ? '' : (' ' + post_name)
        order_shipping_subtotal_amount = 0
        order_shipping_discount_amount = 0
          
        result << "[#{i18n.t('ec_order_code')}] #{self.ec_order_code}"
        result << "================================================================"
        self.order_shippings.each do |order_shipping|
          order_shipping.order_items.each do |order_item|
            case print_locale
            when 'ja'
              print_product_name = order_item.order_product_name_1
            when 'zh_CN'
              product_display = Product.find(:first, :conditions => ["product_code = ? and locale = ?", order_item.product_sku.product.product_code, print_locale],
                :joins =>"inner join product_displays on products.id = product_displays.product_id")
              print_product_name = (product_display.nil?) ? nil : product_display.product_name_1
              print_product_name ||= order_item.order_product_name_2
            else
              print_product_name = order_item.order_product_name_2
            end
            result << "[#{order_item.product_sku.sku_code}] #{print_product_name}"
            order_item_discount = (order_item.discount_amount && order_item.discount_amount > 0) ? order_item.discount_amount : nil
            result << "      #{order_item.unit_price ? order_item.unit_price : 0}#{yen} x #{order_item.qty}#{ ' - %d' % order_item_discount + yen if order_item_discount } = #{order_item.total_amount}#{yen}"
          end
          result << ''
          result << "[#{i18n.t('order_code')}] No.#{order_shipping.order_code}"
          result << "[#{i18n.t('address')}] #{pre_name}#{order_shipping.fullname}#{post_name}"
          result << "  #{i18n.t('zip_code')}#{order_shipping.zip_code}"
          if order_shipping.is_japanese_format?
            result << "  #{order_shipping.address_1}#{order_shipping.address_2}#{order_shipping.address_3}"
          elsif order_shipping.is_chinese_zh_cn_format?
            result << "  #{order_shipping.address_1}#{order_shipping.fulladdress}, #{order_shipping.country.name}"
          else
            result << "  #{order_shipping.fulladdress}, #{order_shipping.address_1}, #{order_shipping.country.name}"
          end
          result << "  (#{i18n.t('phone_number')})#{order_shipping.tel_num}"
          result << "[#{i18n.t('payment_type')}] #{order_shipping.payment_type_div_name}"
          result << "[#{i18n.t('arrival_date')}] #{order_shipping.arrival_date.strftime('%Y/%m/%d') if order_shipping.arrival_date} #{order_shipping.arrival_time_display(print_locale)}"
          result << "================================================================"
          order_shipping_subtotal_amount += order_shipping.subtotal_amount if order_shipping.subtotal_amount
          order_shipping_discount_amount += order_shipping.discount_amount if order_shipping.discount_amount
        end
        result << "#{i18n.t('subtotal')} #{sprintf('%26s',order_shipping_subtotal_amount)}#{yen}"
        result << "#{i18n.t('discount')} #{sprintf('%26s',order_shipping_discount_amount)}#{yen}" if order_shipping_discount_amount > 0
        result << "#{i18n.t('carriage')} #{sprintf('%19s',total_info[:carriage])}#{yen}"
        result << "----------------------------------------------------------------"
        result << "#{i18n.t('total')} #{sprintf('%26s',total_info[:total])}#{yen}"
        result << "----------------------------------------------------------------"
      end
    end
    result << ''
    result.join("\n")
  end

  #
  # = EC注文番号に紐付く商品の小計・送料・合計・個数・送付先数・請求金額を返す =
  #
  # 入力：EC注文見出しID
  # 出力：:subtotal => 小計、:carriage => 送料&手数料、:total => 合計、:count => 個数、:address_count => 送付先数, :due => 請求金額
  #
  def total_info
    result = {:total => 0, :subtotal => 0, :carriage => 0, :count => 0, :address_count => 0, :due => 0}

    self.order_shippings.each do |order_shipping|
      order_shipping.order_items.each do |order_item|
        result[:count] += order_item.qty.to_i
      end
      result[:subtotal] += order_shipping.total_amount
      result[:carriage] += order_shipping.shipping_cost
      unless order_shipping.payment_type_div_entry[:pay_in_advance]
        result[:due] += order_shipping.total_amount + order_shipping.shipping_cost
      end
    end
    result[:total] = result[:subtotal] + result[:carriage]
    result[:address_count] = self.order_shippings.size

    result
  end

  #
  # =在庫をチェックする=
  # 在庫数を超える注文が含まれる場合はOrder::OutOfStockErrorが発生します
  #
  def check_stocks
    self.order_shippings.each do |order_shipping|
      current_order_items = order_shipping.order_items.dup
      current_order_items.each do |order_item|
        order_item.check_stocks
      end
    end
  end

  #
  # =在庫を仮引き当て、引き当て済みにする=
  # 引数(mode)  :paidを指定すると引き当て済みにします。指定しないと仮引き当てにします。
  # 在庫数を超える注文が含まれる場合はOrder::OutOfStockErrorが発生します
  #
  def secure_stocks(mode = :paid)
    raise OrderItem::SecureStockError, 'cannot secure_stocks for new_record' if new_record?
    self.order_shippings.each do |order_shipping|
      current_order_items = order_shipping.order_items.dup
      current_order_items.each do |order_item|
        order_item.secure_stocks(mode)
      end
    end
  end

  #
  # =在庫を開放する=
  #
  def unsecure_stocks
    self.order_shippings.each do |order_shipping|
      current_order_items = order_shipping.order_items.dup
      current_order_items.each do |order_item|
        order_item.unsecure_stocks
      end
    end
  end

  #
  # =ポイントを作成する=
  # 
  # 発行ポイント及び利用ポイントを作成します。
  # 
  #  # 引数(mode)  :paidを指定すると発行ポイントにします。指定しないと仮ポイントにします。
  def create_user_point(given_point = 0, used_point = 0)
    point_setting = PointSetting.find(:first)
    if point_setting.available_flg_true?
      # 発行ポイントがある場合付与ポイントを発行
      unless given_point == 0
        user_point = UserPoint.new({:user_id => user_id, :order_id => id,
            :term_date => point_setting.create_term_date(Date.today), :given_point => given_point})
        user_point.save!
        # 有効期限がポイント発行最終付与時の場合、その会員の保有ポイントの有効期限を付け替える
        UserPoint.term_date_update_for_last_point(user_id, user_point.term_date)
      end
      # 利用ポイントがある場合利用ポイント発行
      unless used_point == 0
        user_point = UserPoint.new({:user_id => user_id, :order_id => id,
            :term_date => point_setting.create_term_date(Date.today), :used_point => used_point})
        user_point.save!
      end
    end
  end

  #
  #  # =仮ポイント付与、ポイント付与にする=
  #  TODO ポイント機能リリース時に利用します。
  #  # 引数(mode)  :paidを指定すると発行ポイントにします。指定しないと仮ポイントにします。
  #  def secure_point(mode = :paid)
  #     user_point = self.user_points.clone
  #     user_point.paid_flg = mode
  #     user_point.updated_at = Date.new
  #     user_point.save!
  #  end

  #
  # =ポイントを削除する=
  #
  # この注文に関連するポイントを全て削除します
  #  TODO ポイント機能リリース時に変更が必要です。
  # #仮ポイント削除時にも利用するよう変更します。
  def delete_point
    self.user_points.each {|point| point.delete}
  end
  
  # 注文に紐付くポイントの合計（獲得ポイント - 利用ポイント）を返す
  def total_point
    given_point - used_point
  end
  
  # 注文に紐付く獲得ポイントを返す
  def given_point
    if self.user_points.nil? || self.user_points.any? {|point| point.given_point < 0}
      # 有効期限切れのポイントを含む場合は0を返す
      return 0
    else
      self.user_points.inject(0) {|result, point| result + point.given_point}
    end
  end
  
  # 注文に紐付く利用ポイントを返す
  def used_point
    if self.user_points.nil? || self.user_points.any? {|point| point.given_point < 0}
      # 有効期限切れのポイントを含む場合は0を返す
      return 0
    else
      self.user_points.inject(0) {|result, point| result + point.used_point}
    end
  end

  #
  # テストデータかどうかを返す
  #
  def test?
    test_flg ? test_flg : false
  end

  #
  # =同送であり、親であるか判定する=
  #
  #親でない場合はnilを返す
  #
  def includable_shipping_parent?
    if self.include_order_id.nil? # 親か同送でない
      include_order = Order.find_by_include_order_id(self.id)
      unless include_order.nil?
        return true
      end
    end
    return nil
  end

  #
  # =ステータス更新=
  #
  def update_status(new_status_div_key = nil)
    self.order_shippings.each do |order_shipping|
      order_shipping.update_status(new_status_div_key)
    end
  end
  
  # 指定した年月に受け付けた注文のうち、配送済み以降のものの件数と注文金額合計を返す
  def self.count_at_term(start_date, end_date)
    statuses = [OrderShipping.status_div_id_by_key(:delivered), OrderShipping.status_div_id_by_key(:deliver_advised)]
    where = <<-"EOS"
      (test_flg IS NULL OR test_flg = 0) 
      AND orders.ordered_at BETWEEN :start_date AND :end_date
      AND order_shippings.status_div IN (:statuses)
    EOS
    conditions = [where, {:start_date => start_date, :end_date => end_date, :statuses => statuses}]
    count = count(:include => :order_shippings, :conditions => conditions)
    amount = sum('order_shippings.total_amount', :include => :order_shippings, :conditions => conditions)
    return {:count => count, :amount => amount}
  end
  
  # 決済オーダーIDを返す
  def payment_order_id
    gmo_transaction_history = GmoTransactionHistory.find(:first, :conditions => ['order_id = ? AND gmo_order_id IS NOT NULL', self.id])
    sps_parameter_history = SpsParameterHistory.find(:first, :conditions => {:order_id => self.id})
    if gmo_transaction_history
      return gmo_transaction_history.gmo_order_id
    elsif sps_parameter_history
      return sps_parameter_history.sps_order_id
    end
  end
end
