# -*- coding: utf-8 -*-
class ProductSkuInventory < ActiveRecord::Base
  versioned

  belongs_to :product_sku

  #その時点での在庫数（子SKUの在庫を考慮しない）を取得する
  def get_stock_count
    ProductSkuInventory.sum('count', :conditions => ['product_sku_id = ? AND id <= ?', self.product_sku_id, self.id])
  end
  
  #履歴登録フォーム
  class CreateForm < ActiveForm
    include SelectableAttr::Base
    include ActiveRecord::Validations
    
    attr_accessor :product_sku_id
    attr_accessor :in_out_div
    attr_accessor :count
    attr_accessor :comment
    
    validates_presence_of :count
    validates_numericality_of :count, :only_integer =>true, :greater_than => 0
    validates_length_of :comment, :maximum=>255, :allow_nil => true
    
    def validate
      #在庫を減らす場合、新しい在庫が現在の引き当て数よりも少なくなってはいけない
      return true if self.in_out_div == 'in' || self.product_sku_id.nil?
      product_sku = ProductSku.find(self.product_sku_id)
      if (product_sku.secure_stock_count > (product_sku.parent_stock_count - self.count.to_i))
        errors.add(:count, :EM200003)
        return false
      end
      return true
    end
    
    def create_inventory_history
      inventory = ProductSkuInventory.new()
      inventory.product_sku_id = self.product_sku_id
      inventory.comment = self.comment
      inventory.count = (self.in_out_div == 'in') ?  self.count.to_i : (-1 * self.count.to_i)
      inventory.save
    end
  end
  
  #在庫管理検索フォーム（ProductSkuごとの在庫数一覧を検索するフォーム）
  class SearchForm < EcRider::ConditionForm
    include SelectableAttr::Base

    attr_accessor :product_code # 商品コード
    attr_accessor :product_name # 商品名
    attr_accessor :stock_count_min  # 在庫数（最小）
    attr_accessor :stock_count_max  # 在庫数（最大）
    attr_accessor :orderable_count_min # 販売可能数（最小）
    attr_accessor :orderable_count_max # 販売可能数（最大）
    attr_accessor :out_of_stock_flg # 在庫切れのみ表示する
    attr_accessor :out_of_safety_stock_flg # 安全在庫切れのみ表示する

    def create_query(options={})
      #product_componentごとの在庫数、引き当て数、構成可能数を抽出する
      #＜抽出カラム名＞
      #id, product_sku_id, component_sku_id, count, child_stock_count, child_secure_stock_count, composable_count, composable_orderable_count
      select_stock_count_from_product_components = <<-"EOS"
        SELECT
          subquery_pc.*,
          /*--- 構成可能数(composable_count) ---*/
          (subquery_pc.child_stock_count DIV subquery_pc.count) AS composable_count,
          /*--- 構成可能数(引き当て反映済み)(composable_orderable_count) ---*/
          ((subquery_pc.child_stock_count - subquery_pc.child_secure_stock_count) DIV count) AS composable_orderable_count
        FROM (
          SELECT 
            pc.id,
            pc.product_sku_id,
            pc.component_sku_id,
            pc.count,
            COALESCE((SELECT SUM(count) FROM product_sku_inventories psi WHERE psi.product_sku_id = pc.component_sku_id), 0) 
              AS child_stock_count,
            COALESCE((SELECT SUM(count) FROM product_sku_secure_inventories pssi WHERE pssi.deleted_at IS NULL AND pssi.product_sku_id = pc.component_sku_id), 0) 
              AS child_secure_stock_count
          FROM product_components pc
        ) subquery_pc
      EOS
      
      #product_skuごとの親SKUとしての在庫数、引き当て数、子SKUからの構成可能数を抽出するSQL
      #＜抽出カラム名＞
      #id, product_id, sku_code, safety_stock_count, parent_stockable_flg, variation_item_1_id, variation_item_2_id, variation_item_3_id,
      #parent_sku_stock_count, sku_secure_stock_count, sku_unpaid_secure_stock_count, component_parent_sku_id, composable_count, composable_orderable_count
      select_parent_stock_count_from_product_skus = <<-"EOS"
        SELECT
          ps.id,
          ps.product_id,
          ps.sku_code,
          ps.safety_stock_count,
          ps.parent_stockable_flg,
          ps.variation_item_1_id,
          ps.variation_item_2_id,
          ps.variation_item_3_id,
          sc.id AS component_parent_sku_id,
          /*--- 親SKUとしての在庫数(parent_sku_stock_count) ---*/
          COALESCE((SELECT SUM(count) FROM product_sku_inventories psi WHERE psi.product_sku_id = ps.id), 0) 
            AS parent_sku_stock_count,
          /*--- 引き当て数(sku_secure_stock_count) ---*/
          COALESCE((SELECT SUM(count) FROM product_sku_secure_inventories pssi WHERE pssi.deleted_at IS NULL AND pssi.product_sku_id = ps.id AND pssi.paid_flg = 1), 0) 
            AS sku_secure_stock_count,
          /*--- 仮引き当て数(sku_unpaid_secure_stock_count) ---*/
          COALESCE((SELECT SUM(count) FROM product_sku_secure_inventories pssi WHERE pssi.deleted_at IS NULL AND pssi.product_sku_id = ps.id AND pssi.paid_flg = 0), 0)
            AS sku_unpaid_secure_stock_count,
          /*--- 子SKUからの構成可能数(composable_count) ---*/
          MIN(sc.composable_count) AS composable_count,
          /*--- 子SKUからの構成可能数(引き当て反映済み)(composable_orderable_count) ---*/
          MIN(sc.composable_orderable_count) AS composable_orderable_count
        FROM product_skus ps
        LEFT JOIN (#{select_stock_count_from_product_components}) sc ON sc.product_sku_id = ps.id
        WHERE
          ps.deleted_at IS NULL
        GROUP BY ps.id
      EOS
      
      #product_skuごとに親子関係を考慮した在庫数、引き当て数、販売可能数を抽出するSQL
      #＜抽出カラム名＞
      #id, product_id, sku_code, safety_stock_count, parent_stockable_flg, variation_item_1_id, variation_item_2_id, variation_item_3_id,
      #parent_sku_stock_count, sku_secure_stock_count, sku_unpaid_secure_stock_count, component_parent_sku_id, composable_count, composable_orderable_count
      #product_code, order_product_name_1, sku_variation_name, sku_stock_count, sku_orderable_count,
      query = <<-"EOS"
        SELECT
          skus.*
        FROM (
          SELECT
            psc.*,
            p.product_code,
            p.order_product_name_1,
            /*--- バリエーション名(「LL - 赤」など)(sku_variation_name) ---*/
            SUBSTRING(CONCAT(
              IF(pvi1.name IS NULL, '', CONCAT(' - ', pvi1.name)),
              IF(pvi2.name IS NULL, '', CONCAT(' - ', pvi2.name)),
              IF(pvi3.name IS NULL, '', CONCAT(' - ', pvi3.name))), 4) AS sku_variation_name,
            /*--- 構成の親子関係を考慮した在庫数(sku_stock_count) ---*/
            IF(psc.component_parent_sku_id IS NULL,
              parent_sku_stock_count,
              IF(psc.parent_stockable_flg = 1,
                LEAST(parent_sku_stock_count, composable_count),
                composable_count)) AS sku_stock_count,
            /*--- 構成の親子関係を考慮した販売可能数(sku_orderable_count) ---*/
            IF(psc.component_parent_sku_id IS NULL,
              (parent_sku_stock_count - sku_secure_stock_count - sku_unpaid_secure_stock_count),
              IF(psc.parent_stockable_flg = 1,
                LEAST(composable_orderable_count, (parent_sku_stock_count - sku_secure_stock_count - sku_unpaid_secure_stock_count)),
                composable_orderable_count)) AS sku_orderable_count
          FROM (#{select_parent_stock_count_from_product_skus}) psc 
          INNER JOIN products p ON psc.product_id = p.id AND p.deleted_at IS NULL
          LEFT JOIN product_variation_item_displays pvi1 ON pvi1.product_variation_item_id = psc.variation_item_1_id AND pvi1.locale = 'ja'
          LEFT JOIN product_variation_item_displays pvi2 ON pvi2.product_variation_item_id = psc.variation_item_2_id AND pvi2.locale = 'ja'
          LEFT JOIN product_variation_item_displays pvi3 ON pvi3.product_variation_item_id = psc.variation_item_3_id AND pvi3.locale = 'ja'
        ) skus
      EOS
      
      where = []
      param = []
      
      # 商品コード/SKUコード
      unless product_code.blank?
        where << "sku_code LIKE ?"
        param << '%'+product_code+'%'
      end
      # 商品名/バリエーション名
      unless product_name.blank?
        where << "(order_product_name_1 LIKE ? OR sku_variation_name LIKE ?)"
        param << '%'+product_name+'%'
        param << '%'+product_name+'%'
      end
      # 在庫数（最小）
      unless stock_count_min.blank?
        where << "sku_stock_count >= ?"
        param << stock_count_min
      end
      # 在庫数（最大）
      unless stock_count_max.blank?
        where << "sku_stock_count <= ?"
        param << stock_count_max
      end
      # 販売可能数（最小）
      unless orderable_count_min.blank?
        where << "sku_orderable_count >= ?"
        param << orderable_count_min
      end
      # 販売可能数（最大）
      unless orderable_count_max.blank?
        where << "sku_orderable_count <= ?"
        param << orderable_count_max
      end
      # 在庫切れのみ表示する
      if out_of_stock_flg == '1'
        where << "sku_stock_count = 0"
      end
      # 安全在庫切れのみ表示する
      if out_of_safety_stock_flg == '1'
        where << "sku_stock_count < safety_stock_count"
      end
      # 検索条件を追加する
      unless where.empty?
        query << " WHERE "
        query << where.join(' AND ')
      end
      # 並び順
      query << ' ORDER BY product_code, sku_variation_name'
      
      return [query] + param
    end
    
    def get_per_page
      per_page = self.disp_count.to_i
      per_page = (per_page.to_i == 0) ? 20 : per_page
    end
  end
  
end
