# -*- coding: utf-8 -*-
class UserPoint < ActiveRecord::Base
  versioned

  belongs_to  :user
  belongs_to  :order

  # 日付対象検索区分
  selectable_attr :date_search_div do
    entry '0', :member_regist, '会員登録日'
    entry '1', :order_date, '注文日'
    entry '2', :term_date, 'ポイント有効期限'
  end

  validates_presence_of :given_point, :if => :user_point_create
  validates_numericality_of :given_point, :only_integer => true, :allow_nil => true
  validates_numericality_of :used_point, :only_integer => true, :allow_nil => true

  # 新規登録かどうかを判断するアクセッサー
  attr_accessor :new_flg
  # ポイント新規投入か否かを判別します。新規投入の場合はtrueを返却します。
  def user_point_create
    return !self.new_flg.nil? && self.new_flg == true
  end
  # 会員ごとの保有ポイントを返却します。
  def self.total_point_for_user(user_id)
    user_point = UserPoint.find(:first, :conditions => ['user_id = ? ', user_id], :select => '(sum(given_point) - sum(used_point)) as total_point')
    return user_point.total_point
  end

  # 会員ごとの有効期限切れでない有効期限を返却します。
  def self.term_date_for_user(user_id)
    user_point = UserPoint.find(:first, :conditions => ['expired_flg = 0 AND term_date IS NOT NULL AND user_id = ? ', user_id], :order => 'term_date')
    unless user_point.nil?
      return user_point.term_date
    end
  end

  # 指定した有効期限まで有効なポイントを返却します。
  def self.point_to_term_date(user_id, term_date)
    user_point = UserPoint.find(:first, :select => '(sum(given_point) - sum(used_point)) as total_point', 
      :conditions => ['expired_flg = 0 AND (used_point != 0 OR (given_point != 0 AND term_date < DATE(?))) AND user_id = ? ',term_date , user_id])
    unless user_point.nil?
      return user_point.total_point
    end
  end


  # 注文、ポイント追加等でポイントを付与するとき、
  # 付与した日付より最終ポイント付与時の有効期限を計算、
  # 有効期限が切れていないポイントのみその有効期限に付け替えます。
  def self.term_date_update_for_last_point(user_id, new_term_date = nil)
    point_setting = PointSetting.find(:first)
    return if new_term_date.nil?
    return unless point_setting.available_flg
    return unless point_setting.terminable_flg
    # ポイント制度利用区分 あり かつ 有効期限 あり かつ 有効期限基準日区分が最終ポイント発行日の場合以下を実行
    if point_setting.term_div_key == :last_point
      user_points = UserPoint.find(:all, :conditions => ["expired_flg = 0 AND user_id = ? AND given_point != 0 ", user_id])
      user_points.each do |user_point|
        user_point.term_date = new_term_date
        user_point.save!
      end
    end
  end
  
  # バッチ用有効期限切れポイント発行処理メソッド
  # 有効期限の切れたポイントを計算し、有効期限切れポイントとしてポイントをマイナス発行します。
  def self.update_expired
    UserPoint.transaction do
      # バッチ未処理の｢発行ポイント-利用ポイント｣の合計を会員単位で検索する
      query = ["select user_id,(sum(given_point)- sum(used_point)) as total_point from user_points where expired_flg = 0 AND (used_point != 0 OR term_date < DATE(?)) group by user_id", Date.today]
      user_total_point = UserPoint.find_by_sql(query)
      # 期限切れのポイントを新規追加する
      user_total_point.each do |expiration_point|
        unless expiration_point.total_point.nil? || expiration_point.total_point.to_i == 0
          new_expiration_point = UserPoint.new()
          new_expiration_point.user_id = expiration_point.user_id
          new_expiration_point.given_point = ('-' + expiration_point.total_point.to_s).to_i
          new_expiration_point.term_date = Date.today
          new_expiration_point.remark = '有効期限切れ'
          new_expiration_point.expired_flg = true
          new_expiration_point.save!
          # 有効期限切れ発行ポイントを経理テーブルに登録
          Accounting.create_for_point_expire(new_expiration_point)
        end
      end
      # 今回バッチでチェックしたレコードの有効期限切れフラグexpired_flgをtrueに設定
      user_points = UserPoint.find(:all, :conditions => ['expired_flg = 0 AND term_date < DATE(?)', DateTime.now])
      user_points.each do |user_point|
        user_point.expired_flg = true
        user_point.save!
      end
    end
  end
  #
  # = 問合せ検索入力フォーム(管理画面用) =
  #
  class SearchInfo < EcRider::ConditionForm
    attr_accessor :date_search_div, :date_from, :date_to, :member_id, :member_name, :total_point, :terminable_flg

    # 開始日の入力制限(yyyy/MM/dd)
    validates_format_of :date_from, :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => Proc.new { |obj| !obj.date_from.blank? }
    # 終了日の入力制限(yyyy/MM/dd)
    validates_format_of :date_to, :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => Proc.new { |obj| !obj.date_to.blank? }

    #
    # = ポイント検索一覧用のクエリを作成します。 =
    #
    def create_user_point_find_query
      query = <<-QUERY
        select user_points.order_id, user_points.user_id, user_points.given_point, user_points.used_point, total_point,
        user_points.term_date, orders.ec_order_code
        from user_points
        left outer join users on user_points.user_id = users.id
        left outer join orders on user_points.order_id = orders.id
        left outer join (select user_id, (sum(given_point) - sum(used_point)) as total_point from user_points GROUP BY user_id) total
        on user_points.user_id = total.user_id
      QUERY
      where = Condition.block do |c|
        c.filter(:ignore_blank)
        case date_search_div
        when UserPoint.date_search_div_id_by_key(:member_regist)
          # 会員登録日
          c.and("users.created_at", ">=", Time.parse(date_from + " 00:00:00")) unless date_from.blank?
          c.and("users.created_at", "<=", Time.parse(date_to + " 23:59:59")) unless date_to.blank?
        when UserPoint.date_search_div_id_by_key(:order_date)
          # 注文日
          c.and("orders.created_at", ">=", Time.parse(date_from + " 00:00:00")) unless date_from.blank?
          c.and("orders.created_at", "<=", Time.parse(date_to + " 23:59:59")) unless date_to.blank?
        when UserPoint.date_search_div_id_by_key(:term_date)
          # ポイント有効期限
          c.and("user_points.term_date", ">=", Time.parse(date_from + " 00:00:00")) unless date_from.blank?
          c.and("user_points.term_date", "<=", Time.parse(date_to + " 23:59:59")) unless date_to.blank?
        end

        # 会員ID
        c.and("user_points.user_id", "like", "%#{member_id}%" ) unless member_id.blank?
        # 会員名
        c.and do |c1|
          _name = delete_space(member_name)
          c1.or "CONCAT(users.name_last, users.name_first)", "like", "%#{_name}%"
          c1.or "CONCAT(users.name_first, users.name_last)", "like", "%#{_name}%"
          c1.or "CONCAT(users.kana_name_last, users.kana_name_first)", "like", "%#{_name}%"
          c1.or "CONCAT(users.kana_name_first, users.kana_name_last)", "like", "%#{_name}%"
        end if member_name && !(member_name.empty?)

        # 保有ポイント
        c.and(:total_point, total_point) unless total_point.blank?

        # 有効期限切れを含むチェック
        c.and("user_points.expired_flg", terminable_flg)
     end

     sql = [query]
     unless where.nil?
       sql[0] << " where #{where.shift}"
       where.each do |value|
        sql << value
       end
     end
       sql[0] << " ORDER BY user_points.created_at DESC"
      return sql
    end

    private
    #
    # 指定の文字列のスペースを削除します。
    # 半角スペースを削除
    #
    def delete_space(str)
      return nil unless str
      str.gsub(' ', '').gsub('　', '')
    end
  end

end
