# -*- coding: utf-8 -*-
require 'ec_rider'
require 'uri'

module EcRider
  class ConfigurationError < StandardError
  end

  class Config

    attr_accessor :default_ec_admin_url
    attr_accessor :default_ec_front_url

    # ブロック中でのみ設定を書き換えます
    def override(config_name, value)
      backup = send(config_name)
      send("#{config_name}=", value)
      begin
        yield if block_given?
      ensure
        send("#{config_name}=", backup)
      end
    end

    # ec_adminの実行時のパス
    attr_accessor_with_default(:ec_admin_root) do
      @ec_admin_root ||= ENV['ECR_ADMIN_ROOT'] || ENV['EC_RIDER_ADMIN_ROOT'] || (
        ENV['EC_RIDER_HOME'] ?
          File.join(ENV['EC_RIDER_HOME'], 'ec_admin') : File.join(RAILS_ROOT, '..', 'ec_admin')
      )
    end

    # ec_frontの実行時のパス
    attr_accessor_with_default(:ec_front_root) do
      @ec_front_root ||= ENV['ECR_FRONT_ROOT'] || ENV['EC_RIDER_FRONT_ROOT'] || (
        ENV['EC_RIDER_HOME'] ?
          File.join(ENV['EC_RIDER_HOME'], 'ec_front') : File.join(RAILS_ROOT, '..', 'ec_front')
      )
    end

    # ec_testの実行時のパス
    attr_accessor_with_default(:ec_test_root) do
      @ec_test_root ||= ENV['ECR_TEST_ROOT'] || ENV['EC_RIDER_TEST_ROOT'] || (
        ENV['EC_RIDER_HOME'] ?
          File.join(ENV['EC_RIDER_HOME'], 'ec_test') : File.join(RAILS_ROOT, '..', 'ec_test')
      )
    end

    # ec_adminの基本URL
    attr_accessor_with_default(:ec_admin_url) do
      (@ec_admin_url ||= ENV['ECR_ADMIN_URL'] || ENV['EC_RIDER_ADMIN_URL'] || @default_ec_admin_url) or
        raise no_config(:ec_admin_url, "set 'ECR_ADMIN_URL' environment variable")
    end

    # ec_frontの基本URL
    attr_accessor_with_default(:ec_front_url) do
      (@ec_front_url ||= ENV['ECR_FRONT_URL'] || ENV['EC_RIDER_FRONT_URL'] || @default_ec_front_url) or
        raise no_config(:ec_front_url, "set 'ECR_FRONT_URL' environment variable")
    end

    # ec_testの基本URL
    attr_accessor_with_default(:ec_test_url) do
      (@ec_test_url ||= ENV['ECR_TEST_URL'] || ENV['EC_RIDER_TEST_URL'] || @default_ec_test_url) or
        raise no_config(:ec_test_url, "set 'ECR_TEST_URL' environment variable")
    end

    # ec_adminのHTTPSでアクセスする際のURL
    attr_accessor_with_default(:ec_admin_https_url) do
      # 設定する項目はありませんが、他の設定と密接に関係するので、ここに定義しておきます。
      @https_uri ||= (
        http_uri = URI(ec_admin_url)
        URI::HTTPS.build(
          :userinfo => http_uri.userinfo,
          :host => http_uri.host,
          :port => self.ec_admin_https_port,
          :path => http_uri.path,
          :query => http_uri.fragment,
          :fragment => http_uri.fragment
        ).to_s
      )
    end

    # ec_adminのHTTPSでアクセスする際のポート
    attr_accessor_with_default(:ec_admin_https_port) do
      @ec_admin_https_port ||= (ENV['ECR_ADMIN_HTTPS_PORT'] || 8443)
    end

    # ec_testとして起動しているか
    attr_accessor_with_default(:ec_test) do
      if File.exists?(File.join(RAILS_ROOT, 'config', 'ec_front.yml'))
        test_environment = YAML::load_file(File.join(RAILS_ROOT, 'config', 'ec_front.yml'))
        test_environment["ec_test"]
      else
        false
      end
    end

    # ec_frontとして起動しているか
    attr_accessor_with_default(:ec_front) do
      if File.exists?(File.join(RAILS_ROOT, 'config', 'ec_front.yml'))
        test_environment = YAML::load_file(File.join(RAILS_ROOT, 'config', 'ec_front.yml'))
        !test_environment["ec_test"]
      else
        false
      end
    end

    # ec_adminとして起動しているか
    attr_accessor_with_default(:ec_admin) do
      !File.exists?(File.join(RAILS_ROOT, 'config', 'ec_front.yml'))
    end

    # ec_frontのBASIC認証アカウント。ステージングで起動したときのために定義しておきます。
    attr_accessor_with_default(:ec_front_basic_auth_account) do
      @ec_front_basic_account = ENV['ECR_FRONT_BASIC_AUTH_ACCOUNT']
    end

    # ec_frontのBASIC認証パスワード。ステージングで起動したときのために定義しておきます。
    attr_accessor_with_default(:ec_front_basic_auth_password) do
      @ec_front_basic_password = ENV['ECR_FRONT_BASIC_AUTH_PASSWORD']
    end

    # ec_testのBASIC認証アカウント
    attr_accessor_with_default(:ec_test_basic_auth_account) do
      @ec_test_basic_account = ENV['ECR_TEST_BASIC_AUTH_ACCOUNT']
    end

    # ec_testのBASIC認証パスワード
    attr_accessor_with_default(:ec_test_basic_auth_password) do
      @ec_test_basic_password = ENV['ECR_TEST_BASIC_AUTH_PASSWORD']
    end

    # このアプリケーションがサポートしているロケールを指定します
    attr_accessor_with_default(:ec_locale) do
      @ec_locale = (ENV['ECR_LOCALE'] ? ENV['ECR_LOCALE'].split(' ') : %w(ja ko zh_CN zh_TW en th))
    end

    # CMSのgitで管理するディレクトリ。.gitがあるディレクトリ
    attr_accessor_with_default(:cms_git_dir) do
      @cms_git_dir ||= ENV['ECR_CMS_GIT_DIR']
    end

    # 障害発生時のエラーメール送付先
    attr_accessor_with_default(:error_mail_from_address, %w(xxxxxx@xxxxxxxx.com))

    # ユーザー,注文などのCSVを保存しておくディレクトリの親(になるかもしれない)ディレクトリ
    attr_accessor_with_default(:csv_dir) do
      ENV['ECR_CSV_DIR'] || @csv_dir || File.join(ec_admin_root, 'tmp', 'csv_file_download')
    end

    # CSVアップロードのエラーCSVを保存しておくディレクトリ
    attr_accessor_with_default(:csv_upload_dir) do
      ENV['ECR_CSV_UPLOAD_DIR'] || @csv_upload_dir || File.join(ec_admin_root, 'tmp', 'csv_file_upload')
    end

    # ユーザーCSVを保存しておくディレクトリ
    attr_accessor_with_default(:csv_users_dir) do
      (ENV['ECR_CSV_USERS_DIR'] || @csv_users_dir) or
        (csv_dir ? File.join(csv_dir, 'users') : nil) or
        raise no_config(:csv_users_dir, "set 'ECR_CSV_USERS_DIR' environment variable")
    end

    # 注文CSVを保存しておくディレクトリ
    attr_accessor_with_default(:csv_orders_dir) do
      (ENV['ECR_CSV_ORDERS_DIR'] || @csv_orders_dir) or
        (csv_dir ? File.join(csv_dir, 'orders') : nil) or
        raise no_config(:csv_orders_dir, "set 'ECR_CSV_ORDERS_DIR' environment variable")
    end

    # お問い合わせ画面の確認用画像を保存しておくディレクトリ
    #
    attr_accessor_with_default(:tmp_inquiry_images_dir) do
      ENV['ECR_TMP_INQUIRY_IMAGES_DIR'] || @tmp_inquiry_images_dir || File.join(RAILS_ROOT, 'public', 'tmp_inquiry_images')
    end

    # productionで動作させる環境でSSLを無効にしたい場合には、この環境変数を設定してください
    attr_accessor_with_default(:ssl_ignored) do
      @ssl_ignored ||= !!(ENV['ECR_SSL_IGNORED'] =~ /true/i)
    end

    # 結合テスト用の共通テーブル
    attr_accessor_with_default(:team_testing_enabled) do
      ActiveRecord::Base.configurations['team_testing'] && ENV['RAILS_ENV'] != 'test'
    end

    # CloudFrontのURL
    attr_accessor_with_default(:cloud_front_url) do
      @cloud_front_url ||= ENV['ECR_CLOUD_FRONT_URL']
    end

    # Amazon Simple Storage Service (S3)のACCESS_KEY_ID
    attr_accessor_with_default(:s3_access_key_id) do
      @s3_access_key_id ||= ENV['ECR_S3_ACCESS_KEY_ID']
    end

    # Amazon Simple Storage Service (S3)のURL
    attr_accessor_with_default(:s3_secret_access_key) do
      @s3_secret_access_key ||= ENV['ECR_S3_SECRET_ACCESS_KEY']
    end

    # Amazon Simple Storage Service (S3)のBucket名
    attr_accessor_with_default(:s3_bucket_name) do
      @s3_bucket_name ||= ENV['ECR_S3_BUCKET_NAME']
    end

    # 決済システムID（1:GMO, 2:SPS）
    attr_accessor_with_default(:payment_system_div) do
      @payment_system_div ||= ENV['PAYMENT_SYSTEM_DIV'] if ENV['RAILS_ENV'] == 'production'
      @payment_system_div ||= '2'
    end

    # GMO用ショップID
    attr_accessor_with_default(:gmo_shop_id) do
      if ENV['RAILS_ENV'] == 'production'
        @gmo_shop_id ||= EcRider.config.ec_test ? ENV['ECR_TEST_GMO_SHOP_ID'] : ENV['ECR_GMO_SHOP_ID']
      end
      @gmo_shop_id ||= 'XXXXXXXXXXX'
    end

    # GMO用ショップパスワード
    attr_accessor_with_default(:gmo_shop_password) do
      if ENV['RAILS_ENV'] == 'production'
        @gmo_shop_password ||= EcRider.config.ec_test ? ENV['ECR_TEST_GMO_SHOP_PASSWORD'] : ENV['ECR_GMO_SHOP_PASSWORD']
      end
      @gmo_shop_password ||= 'XXXXXXXXXXX'
    end

    # GMO用サイトID
    attr_accessor_with_default(:gmo_site_id) do
      if ENV['RAILS_ENV'] == 'production'
        @gmo_site_id ||= EcRider.config.ec_test ? ENV['ECR_TEST_GMO_SITE_ID'] : ENV['ECR_GMO_SITE_ID']
      end
      @gmo_site_id ||= 'XXXXXXXXXXX'
    end

    # GMO用サイトパスワード
    attr_accessor_with_default(:gmo_site_password) do
      if ENV['RAILS_ENV'] == 'production'
        @gmo_site_password ||= EcRider.config.ec_test ? ENV['ECR_TEST_GMO_SITE_PASSWORD'] : ENV['ECR_GMO_SITE_PASSWORD']
      end
      @gmo_site_password ||= 'XXXXXXXXXXX'
    end

    # GMO用プロトコル接続URL
    attr_accessor_with_default(:gmo_uri) do
      if ENV['RAILS_ENV'] == 'production'
        @gmo_uri ||= EcRider.config.ec_test ? ENV['ECR_TEST_GMO_URI'] : ENV['ECR_GMO_URI']
      end
      @gmo_uri ||= 'XXXXXXXXXXX'
    end

    # GMO用プロトコル接続ポート
    attr_accessor_with_default(:gmo_port) do
      if ENV['RAILS_ENV'] == 'production'
        @gmo_port ||= EcRider.config.ec_test ? ENV['ECR_TEST_GMO_PORT'] : ENV['ECR_GMO_PORT']
      end
      @gmo_port ||= 443
    end


    # SPS決済要求URL
    attr_accessor_with_default(:sps_uri) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_uri ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_URI'] : ENV['ECR_SPS_URI']
      end
      @sps_uri ||= 'http://xxxxx.xxxxx.jp/xxx/xxxxxxx'
    end

    # SPSマーチャントID
    attr_accessor_with_default(:sps_merchant_id) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_merchant_id ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_MERCHANT_ID'] : ENV['ECR_SPS_MERCHANT_ID']
      end
      @sps_merchant_id ||= 'XXXXXXX'
    end

    # SPSサービスID
    attr_accessor_with_default(:sps_service_id) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_service_id ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_SERVICE_ID'] : ENV['ECR_SPS_SERVICE_ID']
      end
      @sps_service_id ||= 'XXXXXXX'
    end

    # SPSリクエスト許容時間（デフォルト値：1800（30分））
    attr_accessor_with_default(:sps_limit_second) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_limit_second ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_LIMIT_SECOND'] : ENV['ECR_SPS_LIMIT_SECOND']
      end
      @sps_limit_second ||= 1800
    end

    # SPSのハッシュ生成用のキー
    attr_accessor_with_default(:sps_hash_key) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_hash_key ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_HASH_KEY'] : ENV['ECR_SPS_HASH_KEY']
      end
      @sps_hash_key ||= 'XXXXXXXXXXXXXXXXXXXXX' # 接続支援サイト、ステージング
    end

    # SPS決済完了時URL
    attr_accessor_with_default(:sps_success_url) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_success_url ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_SUCCESS_URL'] : ENV['ECR_SPS_SUCCESS_URL']
      end
      @sps_success_url ||= 'http://xxxxx.xxxxx.jp/xxx/xxxxxxx'
    end

    # SPS決済キャンセル時URL
    attr_accessor_with_default(:sps_cancel_url) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_cancel_url ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_CANCEL_URL'] : ENV['ECR_SPS_CANCEL_URL']
      end
      @sps_cancel_url ||= 'http://xxxxx.xxxxx.jp/xxx/xxxxxxx'
    end

    # SPSエラー時URL
    attr_accessor_with_default(:sps_error_url) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_error_url ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_ERROR_URL'] : ENV['ECR_SPS_ERROR_URL']
      end
      @sps_error_url ||= 'http://xxxxx.xxxxx.jp/xxx/xxxxxxx'
    end

    # SPS決済通知用CGI
    attr_accessor_with_default(:sps_pagecon_url) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_pagecon_url ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_PAGECON_URL'] : ENV['ECR_SPS_PAGECON_URL']
      end
      @sps_pagecon_url ||= 'http://xxxxx.xxxxx.jp/xxx/xxxxxxx'
    end

    # SPSクレジット決済（API接続先ホスト）
    attr_accessor_with_default(:sps_api_host) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_api_host ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_API_HOST'] : ENV['SPS_API_HOST']
      end
      @sps_api_host ||= 'xxxxx.xxxxx.com'
    end

    # SPSクレジット決済（API接続先パス）
    attr_accessor_with_default(:sps_api_path) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_api_path ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_API_PATH'] : ENV['SPS_API_PATH']
      end
      @sps_api_path ||= '/xxxxx/xxxxx'
    end

    # SPSクレジット決済（Basic認証ID）
    attr_accessor_with_default(:sps_basic_auth_id) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_basic_auth_id ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_BASIC_AUTH_ID'] : ENV['SPS_BASIC_AUTH_ID']
      end
      @sps_basic_auth_id ||= 'XXXXXXXXXX'
    end

    # SPSクレジット決済（Basic認証Password）
    attr_accessor_with_default(:sps_basic_auth_password) do
      if ENV['RAILS_ENV'] == 'production'
        @sps_basic_auth_password ||= EcRider.config.ec_test ? ENV['ECR_TEST_SPS_BASIC_AUTH_PASSWORD'] : ENV['SPS_BASIC_AUTH_PASSWORD']
      end
      @sps_basic_auth_password ||= 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'
    end

    # SBPSクレジット決済（proxyサーバのIPアドレス）
    attr_accessor_with_default(:sbps_proxy_ip_address) do
      @sbps_proxy_ip_address ||= ENV['SBPS_PROXY_IP_ADDRESS']
    end

    # SBPSクレジット決済（proxyサーバのポート）
    attr_accessor_with_default(:sbps_proxy_port) do
      @sbps_proxy_port ||= ENV['SBPS_PROXY_PORT']
    end

    # SBPSクレジット決済（proxyサーバの認証ID）
    attr_accessor_with_default(:sbps_proxy_auth_id) do
      @sbps_proxy_auth_id ||= ENV['SBPS_PROXY_AUTH_ID']
    end

    # SBPSクレジット決済（proxyサーバの認証Password）
    attr_accessor_with_default(:sbps_proxy_auth_password) do
      @sbps_proxy_auth_password ||= ENV['SBPS_PROXY_AUTH_PASSWORD']
    end

    # 簡体字入力許可
    attr_accessor_with_default(:chinese_format_using) do
      @chinese_format_using ||= ENV['CHINESE_FORMAT_USING']
      raise no_config(:chinese_format_using, "set 'CHINESE_FORMAT_USING' environment variable")
    end

    private

    def no_config(name, msg)
      ConfigurationError.new("no setting for '#{name.to_s}': #{msg}")
    end
  end
end
