# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe Campaign do
  before(:each) do
    @valid_attributes = {
      :code => "mycode",
      :name => "value for name",
      :main_page_url => "/campaign_page",
      :started_at => Time.now,
      :finished_at => Time.now
    }
  end

  it "should create a new instance given valid attributes" do
    Campaign.delete_all
    Campaign.create!(@valid_attributes)
  end

  describe "availables" do
    fixtures :campaigns
    it "現在日時でキャンペーンを取得" do
      time_now = Time.parse("2009/02/01 00:00:00")
      Time.stub!(:now).and_return(time_now)
      campaigns = Campaign.availables
      campaigns.length.should == 2
    end
    
    it "指定した日時でキャンペーンを取得" do
      time_now = Time.parse("2009/02/01 00:00:00")
      Time.stub!(:now).and_return(time_now)
      order_at = Time.parse("2009/08/01 12:00:00")
      campaigns = Campaign.availables(order_at)
      campaigns.length.should == 5
    end
  end
  
  describe "build_conditions and match_to_conditions?" do
    describe "毎週日曜日のお昼はいいとも特大号特集" do
      before(:each) do 
        @campaign = Campaign.new
        @campaign.conditions.build(:type_key => :now_day_of_week, :operator_key => :eql, :value => '日')
        @campaign.build_conditions.should == "Date.today.cwday.to_i == CampaignCondition.parse_day_of_week('日')"
      end
      it "match" do
        Date.should_receive(:today).and_return(Date.new(2009, 7, 12))
        @campaign.match_to_conditions?.should == true
      end
      it "unmatch" do
        Date.should_receive(:today).and_return(Date.new(2009, 7, 11))
        @campaign.match_to_conditions?.should == false
      end
    end

    describe "【女性限定】お誕生月キャンペーン" do
      before(:each) do 
        @campaign = Campaign.new
        @campaign.conditions.build(:type_key => :user_gender, :operator_key => :eql, :value => 'female')
        @campaign.conditions.build(:type_key => :user_birth_month, :operator_key => :eql, :value => 'today', :connector_key => :and)
        @campaign.build_conditions.should == "user.sex_key.to_s == 'female' && user.birth_month.to_i == CampaignCondition.parse_month('today')"
      end
      it "match" do
        Date.should_receive(:today).and_return(Date.new(2009, 7, 12))
        @campaign.match_to_conditions?(:user => User.new(:sex_key => :female, :birth_month => 7)).should == true
      end
      it "unmatch" do
        Date.should_receive(:today).and_return(Date.new(2009, 7, 11))
        @campaign.match_to_conditions?(:user => User.new(:sex_key => :male, :birth_month => 7)).should == false
        @campaign.match_to_conditions?(:user => User.new(:sex_key => :male, :birth_month => 8)).should == false
        @campaign.match_to_conditions?(:user => User.new(:sex_key => :female, :birth_month => 9)).should == false
      end
    end

    describe "【女性限定】毎月の誕生日はお得キャンペーン" do
      before(:each) do 
        @campaign = Campaign.new
        @campaign.conditions.build(:type_key => :user_gender, :operator_key => :eql, :value => 'female')
        @campaign.conditions.build(:type_key => :user_birth_day, :operator_key => :eql, :value => 'today', :connector_key => :and)
        @campaign.build_conditions.should == 
          "user.sex_key.to_s == 'female'" << 
          " && user.birth_day.to_i == CampaignCondition.parse_day_of_month('today')"
      end
      it "match" do
        Date.should_receive(:today).and_return(Date.new(2009, 7, 12))
        @campaign.match_to_conditions?(:user => User.new(:sex_key => :female, :birth_day => 12)).should == true
      end
      it "unmatch" do
        Date.should_receive(:today).and_return(Date.new(2009, 7, 11))
        @campaign.match_to_conditions?(:user => User.new(:sex_key => :female, :birth_day => 12)).should == false
      end
      
    end

    describe "【女性限定】毎月の誕生日はお得キャンペーン【夜8時から夜10時まで】" do
      before(:each) do 
        @campaign = Campaign.new
        @campaign.conditions.build(:type_key => :user_gender, :operator_key => :eql, :value => 'female')
        @campaign.conditions.build(:type_key => :user_birth_day, :operator_key => :eql, :value => 'today', :connector_key => :and)
        @campaign.conditions.build(:type_key => :now_time, :operator_key => :gte, :value => '20:00', :connector_key => :and, :brace_begin_count => 1)
        @campaign.conditions.build(:type_key => :now_time, :operator_key => :lte, :value => '22:00', :connector_key => :and, :brace_end_count => 1)
        @campaign.build_conditions.should == 
          "user.sex_key.to_s == 'female'" << 
          " && user.birth_day.to_i == CampaignCondition.parse_day_of_month('today')" <<
          " && (Time.now.strftime('%H:%M') >= CampaignCondition.parse_time_of_day('20:00')" <<
          " && Time.now.strftime('%H:%M') <= CampaignCondition.parse_time_of_day('22:00'))"
      end
      it "match" do
        Date.should_receive(:today).and_return(Date.new(2009, 7, 12))
        Time.should_receive(:now).exactly(4).times.and_return(Time.local(2009, 7, 12, 21, 10))
        @campaign.match_to_conditions?(:user => User.new(:sex_key => :female, :birth_day => 12)).should == true
      end
      it "unmatch" do
        Date.should_receive(:today).and_return(Date.new(2009, 7, 12))
        Time.should_receive(:now).exactly(4).times.and_return(Time.local(2009, 7, 12, 23, 10))
        @campaign.match_to_conditions?(:user => User.new(:sex_key => :female, :birth_day => 12)).should == false
      end
      
    end

  end
  
  describe "validate" do
    it "valid" do
      attrs = { :code =>"000001", 
        :name => "テスト", 
        :main_page_url => "/campaign_page",
        :started_at => DateTime.parse("2009/7/7 00:00"),
        :finished_at => DateTime.parse("2009/7/7 23:59"), 
        :conditions_attributes =>{
          "0"=>{
            :campaign_id =>"",
            :no => "", 
            :connector_cd => "01", 
            :type_cd => "01",
            :brace_begin_count => "",
            :brace_end_count => "",
            :operator_cd => "05", 
            :value => "male"
          }}}
      Campaign.delete_all
      campaign = Campaign.new(attrs)
      campaign.valid?
      campaign.errors.full_messages.should == []
      campaign.save.should == true
    end

    it "invalid conditions brace_count" do
      attrs = { :code =>"000001", 
        :name => "テスト", 
        :main_page_url => "/campaign_page",
        :started_at => DateTime.parse("2009/7/7 00:00"),
        :finished_at => DateTime.parse("2009/7/7 23:59"), 
        :conditions_attributes =>{
          "0"=>{
            :campaign_id =>"",
            :no => "", 
            :connector_cd => "01", 
            :type_cd => "01",
            :brace_begin_count => "1",
            :brace_end_count => "",
            :operator_cd => "05", 
            :value => "male"
          }}}
      Campaign.delete_all
      campaign = Campaign.new(attrs)
      campaign.valid?
      campaign.errors.full_messages.should == ["条件の括弧の数が合っていません"]
      campaign.save.should == false
    end

    it "invalid conditions brace_count" do
      attrs = { :code =>"000001", 
        :name => "テスト", 
        :main_page_url => "/campaign_page",
        :started_at => DateTime.parse("2009/7/7 00:00"),
        :finished_at => DateTime.parse("2009/7/7 23:59"), 
        :conditions_attributes =>{
          "0"=>{
            :campaign_id =>"",
            :no => "", 
            :connector_cd => "01", 
            :type_cd => "01",
            :brace_begin_count => "",
            :brace_end_count => "1",
            :operator_cd => "05", 
            :value => "male"
          },
          "1"=>{
            :campaign_id =>"",
            :no => "", 
            :connector_cd => "01", 
            :type_cd => "01",
            :brace_begin_count => "1",
            :brace_end_count => "",
            :operator_cd => "05", 
            :value => "male"
          }}}
      Campaign.delete_all
      campaign = Campaign.new(attrs)
      campaign.valid?
      campaign.errors.full_messages.should == ["条件の括弧の数が合っていません"]
      campaign.save.should == false
    end

    it "invalid total conditions brace_count" do
      attrs = { :code =>"000001", 
        :name => "テスト", 
        :main_page_url => "/campaign_page",
        :started_at => DateTime.parse("2009/7/7 00:00"),
        :finished_at => DateTime.parse("2009/7/7 23:59"), 
        :conditions_attributes =>{
          "0"=>{
            :campaign_id =>"",
            :no => "", 
            :connector_cd => "01", 
            :type_cd => "01",
            :brace_begin_count => "1",
            :brace_end_count => "",
            :operator_cd => "05", 
            :value => "male"
          },
          "1"=>{
            :campaign_id =>"",
            :no => "", 
            :connector_cd => "01", 
            :type_cd => "01",
            :brace_begin_count => "",
            :brace_end_count => "2",
            :operator_cd => "05", 
            :value => "male"
          }}}
      Campaign.delete_all
      campaign = Campaign.new(attrs)
      campaign.valid?
      campaign.errors.full_messages.should == ["条件の括弧の数が合っていません"]
      campaign.save.should == false
    end
  end

  describe "with Cart" do
    def no_change_attrs_block(receiver, *attrs, &block)
      attr = attrs.pop
      return block unless attr
      inner = no_change_attrs_block(receiver, *attrs, &block)
      lambda{
        begin
          inner.call
        rescue
          puts "no_change_attrs_block(#{receiver.class.name.inspect}, *#{attrs.inspect}): inner => #{inner.inspect}"
          raise
        end
      }
    end

    fixtures :users, :user_ranks, 
      :products, :product_skus, :product_prices,
      :campaigns, :campaign_conditions, :campaign_effects, :campaign_target_product_skus

    before(:each) do
      @user_rank = UserRank.create(:name => 'カートのテスト用ランク')
      @user = users(:users_00010) # 会員二十三
      @user.user_rank_id = @user_rank.id
      @user.save!
      @user.reload
      @product_skus = {}
      @product_skus[:A] = products(:products_00003).primary_sku # 商品A インテリア家具
      @product_skus[:B] = products(:products_00005).primary_sku # 商品B タンブラー
      @product_skus[:C] = products(:products_00006).primary_sku # 商品C ガム10箱
      prices = {:A => 100000, :B => 10000, :C => 1000}
      prices.each do |name, price|
        ProductPrice.create(
          :product_sku_id => @product_skus[name].id, 
          :user_rank_id => @user_rank.id,
          :standard_price => price,
          :regular_purchase_price => price - 200  # 定期購入売価を標準売価より安くしておいて、テスト対象としてく区別する
          ) 
      end
      @cart = Cart.new
      @cart.user_rank = @user_rank
    end

    # キャンペーンT0001A:  注文合計10,000円以上で10%引き（条件タイプ：２）、優先度100
    describe "Campaign: T0001A" do
      before(:each) do
        time_now = Time.parse("2009/01/01 12:00:00")
        Time.stub!(:now).and_return(time_now)
        campaigns = Campaign.availables
        campaigns.length.should == 1
        @campaign_a = campaigns.first
        @campaign_a.conditions.first.type_key.should == :cart_product_subtotal_price
        @campaign_a.conditions.first.value.should == '10000'
      end
      
      it "match?はマッチするときはtrueを返す" do
        # 合計金額は11,000円なのでマッチする
        @cart.add(@product_skus[:B])
        @cart.add(@product_skus[:C])
        @cart.calculate
        @campaign_a.match?(@cart).should == true
      end

      it "match?はマッチしないときはfalseを返す" do
        # 合計金額は1,000円なのでマッチしない
        @cart.add(@product_skus[:C])
        @cart.calculate
        @campaign_a.match?(@cart).should == false
      end

      it "apply_if_match はマッチしないときには何もしない" do
        # 合計金額は1,000円なのでマッチしない
        @cart.add(@product_skus[:C])
        item = @cart.item_list.first
        lambda{
          no_change_attrs_block(@cart, :subtotal_amount, :discount_amount, :total_amount){ 
            no_change_attrs_block(item, :qty, :unit_price, :unit_price_x_qty, 
              :shipping_cost, :discount_amount, :subtotal_amount){
              @campaign_a.apply_if_match(@cart)
            }.call
          }.call
        }.should_not change(@cart.items, :length)
      end

      it "apply_if_match でマッチする場合" do
        @cart.add(@product_skus[:B])
        @cart.add(@product_skus[:C])
        @cart.calculate
        @campaign_a.apply_if_match(@cart)
        @cart.subtotal_amount.should == 11000
        @cart.discount_amount.should == 1100
        # @cart.total_amount.should == 9900
      end
    end

    # キャンペーンT0002A: 注文合計10,000円以上で10%引き（条件タイプ：２）、優先度100
    # キャンペーンT0002B: 商品Aが50%引き（条件タイプ：１）、優先度80
    describe "T0002" do
      before(:each) do
        time_now = Time.parse("2009/02/01 12:00:00")
        Time.stub!(:now).and_return(time_now)
        campaigns = Campaign.availables
        campaigns.length.should == 2
        @campaign_a = campaigns.first
        @campaign_a.conditions.first.type_key.should == :cart_product_subtotal_price
        @campaign_a.conditions.first.value.should == '10000'
        @campaign_a.targets.empty?.should == true
        effect = @campaign_a.effects.first
        effect.type_key.should == :discount_percentage
        effect.operator_key.should == :assign
        effect.value.should == '10'
        @campaign_b = campaigns.last
        @campaign_b.conditions.empty?.should == true
        @campaign_b.targets.empty?.should == false
        @campaign_b.effects.length.should == 1
        effect = @campaign_b.effects.first
        effect.type_key.should == :discount_percentage
        effect.operator_key.should == :assign
        effect.value.should == '50'
      end

      it "Bは商品Aを持つカートにマッチする" do
        @cart.add(@product_skus[:A])
        @campaign_b.match?(@cart).should == true
      end

      it "Bは商品Aを持つカートに適用される" do
        @cart.add(@product_skus[:A])
        @cart.calculate
        cart_item = @cart.item_for(@product_skus[:A])
        cart_item.subtotal_amount.should == 100000
        cart_item.discount_amount.should ==  50000
        cart_item.total_amount.should    ==  50000
        @cart.subtotal_amount.should     ==  50000
        @cart.discount_amount.should     ==      0
        @cart.total_amount.should        ==  50000
      end
    end
  end

  
  describe "#1180 同一の優先度で重複する期間がある場合はバリデーションエラー" do
    #
    #        0                    1                   2     
    #   |--------------|     |----------|      |--------------|
    #   |              |     |    3     |      |              |
    #   |       |-------------------------------------|       |
    #   |       |      |     |          |      |      |       |
    #   11/24   12/1   12/5  12/7       12/10  12/12  12/16   12/31
    #
    before do
      Campaign.delete_all
      periods = [
        ['2009/11/24 00:00:00', '2009/12/05 23:59:59'],
        ['2009/12/07 00:00:00', '2009/12/10 23:59:59'],
        ['2009/12/12 00:00:00', '2009/12/31 23:59:59'],
        ['2009/12/01 00:00:00', '2009/12/16 23:59:59']
      ]
      @campaigns = []
      periods.each_with_index do |period, idx|
        # どのキャンペーンをsaveするかは各itに任せます。ここではsaveしません。
        @campaigns << Campaign.new(:code => "T1180#{idx}", :name => "#1180-#{idx}", 
          :priority => 1,
          :started_at => period.first, :finished_at => period.last)
      end
    end

    describe "同一priorityで期間が重複しない場合" do
      patterns = [
        [0, 1, 2], [1, 0, 2],
        [1, 2, 0], [2, 1, 0], 
        [2, 0, 1], [0, 2, 1], 
      ]
      patterns.each do |pattern|
        it "#{pattern.inspect}の順" do
          pattern.each{|idx| @campaigns[idx].save!}
        end
      end
    end

    describe "同一priorityで期間が重複する場合" do

      # 12/3 -- 12/14のキャンペーンは0-3の全てのキャンペーンと期間が重複します。
      (0..3).each do |campaign_index|
        it "#{campaign_index}と重複する場合" do
          @campaigns[campaign_index].save!
          campaign = Campaign.new(:code => "T1180X", :name => "#1180X",
            :started_at => "2009/12/03 00:00:00", :finished_at => "2009/12/14 23:59:59")
          campaign.valid?
          campaign.errors.full_messages.should == ["期間が重複する場合、優先度は同じ値を設定することができません。"]
        end
      end
    end

    
  end




end
