# -*- coding: utf-8 -*-
class GmoTransactionHistory < ActiveRecord::Base
  set_table_name 'gmo_transaction_histories'
  acts_as_paranoid

  belongs_to :order, :class_name => 'Order', :foreign_key => 'order_id'

  # 実行区分
  selectable_attr :exe_div do
    entry '1', :first_time, '初回実行'
    entry '2', :again, '再実行'
  end

  # 対応区分
  selectable_attr :status_div do
    entry '0', :no_need, '対応必要なし'
    entry '1', :not_yet, '未対応'
    entry '2', :waiting, '返答待ち'
    entry '3', :complete, '対応完了'
  end

  before_validation :set_test_flg
  def set_test_flg
    self.test_flg = true if self.test_flg.nil? && EcRider.config.ec_test
  end

  def user
    @user ||= self.order.user if self.order
    @user
  end

  def user_id
    self.user.id if self.user
  end

  def locale
    self.order.locale if self.order
  end


  # 検索条件生成
  def self.create_condition(condition)
    where = Condition.block do |c|
      c.filter(:ignore_blank)
      # エラーコード
      c.and "sql", "err_code IS NOT NULL"
      c.and "err_info", "like", "%#{condition.err_code}%"
      # Gpay注文ID
      c.and :gmo_order_id, condition.gmo_order_id
      # 処理ステータス区分
      c.and :status_div, condition.status_div
      # 処理区分
      c.and :exe_div, condition.exe_div
      # 登録日Fromの条件
      c.and "created_at", ">=", condition.exe_date_time_from
      # 登録日Toの条件
      c.and "created_at", "<=", condition.exe_date_time_to
      # テスト決済の条件
      if condition.test_flg == CmsPublish.target_id_by_key(:test)
        c.and("test_flg", "1")
      elsif condition.test_flg == CmsPublish.target_id_by_key(:production)
        c.and("sql", "test_flg IS NULL")
      end
    end
  end

  # エラーコードがシステムエラーの場合にtrueを返す
  def self.is_system_err_code?(error_code)
    !(/^G/ =~ error_code)
  end
  def is_system_err_code?
    GmoTransactionHistory.is_system_err_code?(self.err_code)
  end


  # 検索フォーム
  class SearchConditionForm < ActiveForm
    attr_accessor :exe_date_from, :exe_time_from, :exe_date_to, :exe_time_to, :exe_div, :status_div, :disp_count,
    :gmo_order_id, :exe_result_div, :err_code, :test_flg

    # 日時フォーマット
    validates_format_of :exe_date_from, :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => :exe_date_from_unempty?
    validates_format_of :exe_date_to,   :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => :exe_date_to_unempty?
    validates_format_of :exe_time_from, :with => /^([0-1][0-9]|[2][0-3]):[0-5][0-9]:[0-5][0-9]$/, :message => :EM070002, :if => :exe_time_from_unempty?
    validates_format_of :exe_time_to,   :with => /^([0-1][0-9]|[2][0-3]):[0-5][0-9]:[0-5][0-9]$/, :message => :EM070002, :if => :exe_time_to_unempty?

    def exe_date_from_unempty?
      exe_date_from && !exe_date_from.empty?
    end
    def exe_time_from_unempty?
      exe_time_from && !exe_time_from.empty?
    end
    def exe_date_to_unempty?
      exe_date_to && !exe_date_to.empty?
    end
    def exe_time_to_unempty?
      exe_time_to && !exe_time_to.empty?
    end

    # 実行日時をTime型に変換する（日時が指定されていなければnilを返す）
    def exe_date_time_from
      return nil if !self.exe_date_from || self.exe_date_from.empty?
      from_str = "#{self.exe_date_from} #{self.exe_time_from}"
      return Time.parse(from_str)
    end
    def exe_date_time_to
      return nil if !self.exe_date_to || self.exe_date_to.empty?
      to_str = "#{self.exe_date_to} #{self.exe_time_to}"
      return Time.parse(to_str)
    end

    # 日付の存在チェック
    def validate
      if exe_date_from_unempty?
        _date = exe_date_from.split('/')
        unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
          errors.add(:exe_date_from, :EM990007)
        end
      end
      if exe_date_to_unempty?
        _date = exe_date_to.split('/')
        unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
          errors.add(:exe_date_to, :EM990007)
        end
      end
    end
  end

  #
  # テスト決済かどうかを返す
  #
  def test?
    test_flg ? test_flg : false
  end
end
