# -*- coding: utf-8 -*-
class OrderForm < ActiveForm
  include SelectableAttr::Base
  include ActiveRecord::Validations

  attr_accessor( :send_address_index, :normal_payment_type, :mail_payment_type, 
    :gift_flg, :send_invoice_separately_flg, :write_sender_flg, :free_request,
    :card_kind, :no_1, :no_2, :no_3, :no_4, :expire_year, :expire_month, :card_name,
    :arrival_date, :arrival_hour,
    :sent_one_place, :has_credit, # 複数送付の場合に利用
    :expire, # 有効期限のエラーメッセージ用
    :include_order_id, :delivery_date, :delivery_time)

  # カードの種類
  selectable_attr :card_kind do
    update_with_kbn KbnCode.name_div_id_by_key(:card_kind)
    entry 'visa'  , :visa  , 'Visa'           , :as_amex => false
    entry 'master', :master, 'MasterCard'     , :as_amex => false
    entry 'amex'  , :amex  , 'AmericanExpress', :as_amex => true
    entry 'diners', :diners, 'DinersClub'     , :as_amex => true
    entry 'jcb'   , :jcb   , 'JCB'            , :as_amex => false
  end

  selectable_attr :normal_payment_type, :enum => OrderShipping.payment_type_div_enum
  selectable_attr :mail_payment_type  , :enum => OrderShipping.payment_type_div_enum

  # 必須項目
  validates_presence_of :send_address_index, :if => :sent_one_place

  #文字数バリデーション
  validates_length_of   :free_request, :maximum => 1000, :message => :EM990012, :if => :check_free_request?
  #（半角英数＋半角記号＋半角スペース）
  validates_format_of :free_request, :with => /^[A-Za-z0-9,_\.\-\+\(\)\s]*$/, :message => :EM990028, :if => :is_english_format?

  # カード情報必須チェック
  with_options(:if => :credit_card?) do |card|
    card.validates_presence_of :card_kind, :message => :EM020001
    card.validates_presence_of :expire_year, :message => :EM990001
    card.validates_presence_of :expire_month, :message => :EM990001
    card.validates_presence_of :card_name, :message => :EM990001
  end

  # カード番号チェック（「AMEX」でない場合）
  with_options(:if => :credit_card_not_amex?) do |card|
    card.validates_format_of :no_1, :with => /\d{4}/, :message => :EM990013
    card.validates_format_of :no_2, :with => /\d{4}/, :message => :EM990013
    card.validates_format_of :no_3, :with => /\d{4}/, :message => :EM990013
    card.validates_format_of :no_4, :with => /\d{4}/, :message => :EM990013
  end
  # カード番号チェック（「AMEX」の場合）
  with_options(:if => :credit_card_amex?) do |card|
    card.validates_format_of :no_1, :with => /\d{4}/, :message => :EM990013
    card.validates_format_of :no_2, :with => /\d{6}/, :message => :EM990015
    card.validates_format_of :no_3, :with => /^[0-9]{4,5}$/, :message => :EM990019
  end

  # 有効期限
  validates_format_of :expire_year,  :with => /^[0-9]{2}$/, :message => :EM990033, :if => :expire_year_check?
  validates_format_of :expire_month, :with => /^[0-9]{2}$/, :message => :EM990033, :if => :expire_month_check?
  # 有効期限の存在チェック
  def validate
    if normal_payment_type_key == :credit
      if !expire_year.nil? && !expire_month.nil?
        if !expire_year.empty? || !expire_month.empty?
          unless Date::exist?(expire_year.to_i, expire_month.to_i, 1)
            errors.add(:expire, :EM990007)
          end
        end
      end
    end
  end

  # クレジットカード払いの場合はtrueを返す
  def credit_card?
    (normal_payment_type_key == :credit) || (mail_payment_type_key == :credit) || has_credit
  end
  # クレジットカード払いかつ、「AMEX」「Diners」の場合はtrueを返す
  def credit_card_amex?
    credit_card? && card_kind_entry[:as_amex]
  end
  # クレジットカード払いかつ、「AMEX」「Diners」以外の場合はtrueを返す
  def credit_card_not_amex?
    credit_card? && !card_kind_entry[:as_amex]
  end

  # カード番号を連結します
  def card_no
    [no_1, no_2, no_3, no_4].join
  end

  # 有効期限を「yymm」に整形します
  def expire
    [expire_year, expire_month].join
  end

  # 有効期限が入力されていればtrueを返す
  def expire_year_check?
    !expire_year.blank? && normal_payment_type_key == :credit
  end
  def expire_month_check?
    !expire_month.blank? && normal_payment_type_key == :credit
  end

  # 自由記入欄が入力されていればtrueを返す
  def check_free_request?
    !free_request.blank?
  end

  # 英数チェックする場合はtrueを返す
  def is_english_format?
    I18n.locale.to_s != 'ja'
  end

  #
  # = 配送希望日を返却します =
  #
  def available_delivery_date(country_code = "JP")
    list = [[]] # 配送希望日を未選択にするブランクを設定しています
    Distributor.delivery_date_list(country_code).each do |date|
      list << [date, date]
    end
    return list
  end

  #
  # = 配送希望時間を返却します =
  #
  def available_delivery_time(country_code="JP")
    country = Country.find_by_code(country_code)
    distributor_id = country.distributor_id
    DistributorDeliveryTime.find(:all, :conditions => ["distributor_id = ?", distributor_id])
  end
end
