# -*- coding: utf-8 -*-
class OrderItem < ActiveRecord::Base
  include EcRider::ValidationExt
  versioned
  acts_as_paranoid

  attr_accessor :product_image, :delete_flg

  belongs_to :order_shipping, :class_name => 'OrderShipping', :foreign_key => 'order_shipping_id'
  belongs_to :product_sku, :with_deleted => true

  has_many :product_sku_secure_inventories

  # コース区分
  selectable_attr :course_div do
    entry '1', :normal  , '単品'      # 都度購入
    entry '2', :periodic, '定期'      # 定期購入
    entry '3', :present , 'プレゼント' # キャンペーンによるプレゼント
  end
  
  # validates_length_ofが呼び出すことになるFixnum#sizeは、32bit版OSで動かした時は4を、64bit版OS上で動作した場合には8を返します。
  # 以下のようにvalidates_length_ofでは数値を文字列に変換した上での長さのチェックを行うことはできません。
  # validates_inclusion_ofを使用するか、validateメソッドにチェックするコードを用意してください。
  # validates_length_of   :qty, :maximum=>5, :allow_nil => false
  # validates_length_of   :subtotal_amount, :maximum=> 10, :allow_nil => false
  validates_numeric_max_length_of(:qty, :maximum => 5, :allow_nil => false)
  validates_numeric_max_length_of(:subtotal_amount, :maximum => 10, :allow_nil => false)

  validates_numericality_of :qty, :only_integer =>true, :message => :EM990003
  validates_numericality_of :subtotal_amount, :only_integer =>true, :message => :EM990003

  #
  # =order_itemを保存して在庫を引き当てる=
  # 注文情報編集画面からの注文情報更新時に呼ばれます。
  #
  def save_with_stock_control(status_div_key = nil)
    self.save!
    case status_div_key
      when :undelivery_settlement, :undelivery_unpaid
        self.secure_stocks(:before_pay)
      when :undelivery, :deliver_request
        self.secure_stocks
    end
  end

  #
  # =order_itemを削除して在庫を開放する=
  #
  def destroy_with_stock_control
    self.unsecure_stocks
    self.destroy
  end

  #
  # =在庫をチェックする=
  # 引き当て後の在庫数を返す
  # 在庫数を超える注文が含まれる場合はOrderItem::OutOfStockErrorが発生する
  #
  def check_stocks
    product_sku = ProductSku.find_by_id(self.product_sku_id)
    return unless product_sku

    add_quantity = self.qty

    #引き当て済みの在庫がある場合は差分を計算する
    if secure_stock = ProductSkuSecureInventory.find(:first, :conditions => {:order_item_id => self.id, :product_sku_id => self.product_sku_id})
      add_quantity -= secure_stock.count
    end

    new_stock_count = product_sku.orderable_count - add_quantity
    if new_stock_count < 0
      message = I18n.t("activerecord.errors.messages.EM200002", :product => self.product_name, :count => product_sku.orderable_count)
      raise OutOfStockError.new(self.id, message, self.product_name)
    end

    return new_stock_count
  end
  
  #
  # =在庫を仮引き当て、引き当て済みにする=
  # 引数(mode)  :paidを指定すると引き当て済みにします。指定しないと仮引き当てにします。
  #
  def secure_stocks(mode = :paid)
    raise SecureStockError, 'cannot secure_stocks for new_record' if new_record?
    
    check_stocks
    
    # テスト注文は在庫に関与しない
    return if self.order_shipping.order.test_flg
    unless ::EcRider.config.ec_test
      product_sku = ProductSku.find(self.product_sku_id, :include => :components, :with_deleted => true)
      create_or_update_inventory(mode)
      product_sku.components.each do |component|
        create_or_update_inventory(mode, component.component_sku_id, component.count * self.qty)
      end
    end
  end
  
  #
  # =在庫を減らす=
  #
  def reduce_stocks
    # テスト注文は在庫に関与しない
    return if self.order_shipping.order.test_flg
    order_code = self.order_shipping.order.ec_order_code
    self.product_sku_secure_inventories.each do |secure_inventory|
      inventory = ProductSkuInventory.new(
        :count => -secure_inventory.count, 
        :comment => "オンライン受注番号[#{order_code}]を出荷しました",
        :product_sku_id => secure_inventory.product_sku_id
      )
      inventory.save!
      secure_inventory.destroy
    end
  end
  
  #
  # =在庫を戻す=
  #
  def restore_stocks
    # テスト注文は在庫に関与しない
    return if self.order_shipping.order.test_flg
    order_code = self.order_shipping.order.ec_order_code
    ProductSkuInventory.create!(
      :count => self.qty, 
      :comment => "オンライン受注番号[#{order_code}]を配送指示済に戻しました",
      :product_sku_id => self.product_sku_id
    )
    
    # 子SKUがある場合は子SKUの在庫も戻す
    product_sku = ProductSku.find(self.product_sku_id, :include => :components, :with_deleted => true)
    product_sku.components.each do |component|
      ProductSkuInventory.create!(
        :count => component.count * self.qty, 
        :comment => "オンライン受注番号[#{order_code}]を配送指示済に戻しました",
        :product_sku_id => component.component_sku_id
      )
    end
  end
  
  private
  def create_or_update_inventory(mode = :paid, sku_id = self.product_sku_id, count = self.qty)
    secure_stock = ProductSkuSecureInventory.find(:first, :conditions => {:order_item_id => self.id, :product_sku_id => sku_id})
    secure_stock ||= ProductSkuSecureInventory.new(:order_item_id => self.id, :product_sku_id => sku_id, :count => count)
    secure_stock.count = count
    secure_stock.paid_flg = (mode == :paid)
    secure_stock.save!
  end
  
  public
  #
  # =引き当て在庫を開放する=
  #
  def unsecure_stocks
    # テスト注文は在庫に関与しない
    return if self.order_shipping.order.test_flg
    #引き当てている在庫を開放する
    self.product_sku_secure_inventories.each do |secure_inventory|
      secure_inventory.destroy
    end
  end

  #
  # =在庫切れ例外=
  # 在庫数以上の注文を確定しようとした際に発生する
  #
  class OutOfStockError < StandardError
    attr_reader :order_item_id, :message, :product_name
    def initialize(order_item_id, message = nil, product_name = nil)
      @order_item_id = order_item_id
      @message = message
      @product_name = product_name
    end
  end

  #
  # =引き当て例外=
  # new_recordに対してsecure_stocksを呼び出した際に発生する
  #
  class SecureStockError < StandardError
  end

  # 対象の商品SKUを返す
  # 紐付く商品SKUが無い場合（削除されている場合）は、新規ProductSkuインスタンスを返す
  def get_sku
    return self.product_sku if self.product_sku
    return ProductSku.new()
  end
end
