# -*- coding: utf-8 -*-
class PaymentErrorHistory < ActiveRecord::Base  
  acts_as_paranoid
  
  belongs_to :order, :with_deleted => true
  belongs_to :updatable, :polymorphic => true
  belongs_to :sps_user, :class_name => 'SpsUserTemporary'
  
  # 実行区分
  selectable_attr :exe_div, :enum => GmoTransactionHistory.exe_div_enum
  # ステータス
  selectable_attr :status_div, :enum => GmoTransactionHistory.status_div_enum
  # 決済システム
#  selectable_attr :payment_system_div, :enum => CreditCardPayment.payment_system_div_enum
  
  # 決済方法
  selectable_attr :payment_div do
    entry '1', :gmo, 'クレジット'
    entry '2', :sps, 'Alipay'
  end
  
  # Alipayエラーコード
  selectable_attr :err_code, :enum => SpsResultHistory.ecr_result_enum

  # ユーザ名を返す
  def user_name
    order = self.updatable.order
    user = order.user if order
    user.fullname if user
  end
  
  #
  # 検索フォーム
  #
  class SearchConditionForm < ActiveForm
    attr_accessor :exe_date_from, :exe_time_from, :exe_date_to, :exe_time_to, :exe_div, :status_div, :disp_count,
      :gpay_order_id, :exe_result_div, :err_code, :test_flg

    # 実行日時フォーマット
    validates_format_of :exe_date_from, :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => :exe_date_from_unempty?
    validates_format_of :exe_date_to,   :with => /^\d{4}\/\d{1,2}\/\d{1,2}$/, :message => :EM070001, :if => :exe_date_to_unempty?
    validates_format_of :exe_time_from, :with => /^([0-1][0-9]|[2][0-3]):[0-5][0-9]:[0-5][0-9]$/, :message => :EM070002, :if => :exe_time_from_unempty?
    validates_format_of :exe_time_to,   :with => /^([0-1][0-9]|[2][0-3]):[0-5][0-9]:[0-5][0-9]$/, :message => :EM070002, :if => :exe_time_to_unempty?

    def exe_date_from_unempty?
      exe_date_from && !exe_date_from.empty?
    end
    def exe_time_from_unempty?
      exe_time_from && !exe_time_from.empty?
    end
    def exe_date_to_unempty?
      exe_date_to && !exe_date_to.empty?
    end
    def exe_time_to_unempty?
      exe_time_to && !exe_time_to.empty?
    end

    # 実行日時をTime型で返す
    # 登録日が指定されていない場合はnilを返す。
    def exe_date_time_from
      return nil if !self.exe_date_from || self.exe_date_from.empty?
      from_str = "#{self.exe_date_from} #{self.exe_time_from}"
      return Time.parse(from_str)
    end
    def exe_date_time_to
      return nil if !self.exe_date_to || self.exe_date_to.empty?
      to_str = "#{self.exe_date_to} #{self.exe_time_to}"
      return Time.parse(to_str)
    end

    # 日付の存在チェック
    def validate
      if exe_date_from_unempty?
        _date = exe_date_from.split('/')
        unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
          errors.add(:exe_date_from, :EM990007)
        end
      end
      if exe_date_to_unempty?
        _date = exe_date_to.split('/')
        unless Date::exist?(_date[0].to_i, _date[1].to_i, _date[2].to_i)
          errors.add(:exe_date_to, :EM990007)
        end
      end
    end
  end

  # 検索条件生成
  def self.create_condition(condition)
    where = Condition.block do |c|
      c.filter(:ignore_blank)
      # エラーコード
      c.and "sql", "err_code IS NOT NULL"
      c.and "err_info", "like", "%#{condition.err_code}%"
      # Gpay注文ID
      c.and :gpay_order_id, condition.gpay_order_id
      # 処理ステータス区分
      c.and :status_div, condition.status_div
      # 処理区分
      c.and :exe_div, condition.exe_div
      # 登録日Fromの条件
      c.and "created_at", ">=", condition.exe_date_time_from
      # 登録日Toの条件
      c.and "created_at", "<=", condition.exe_date_time_to
      # テスト決済の条件
      if condition.test_flg == CmsPublish.target_id_by_key(:test)
        c.and("test_flg", "1")
      elsif condition.test_flg == CmsPublish.target_id_by_key(:production)
        c.and("sql", "test_flg IS NULL")
      end
    end
  end

  #
  # テスト決済かどうかを返す
  #
  def test?
    test_flg ? test_flg : false
  end
end
