# -*- coding: utf-8 -*-
class ProductCategory < ActiveRecord::Base
  versioned

  include LocaleDisplayable

  belongs_to :parent, :class_name => "ProductCategory", :foreign_key => 'parent_id'
  has_many :children, :class_name => "ProductCategory", :foreign_key => 'parent_id',
    :order => 'display_order, id', :dependent => :destroy

  has_many :displays, :class_name => "ProductCategoryDisplay", :foreign_key => 'product_category_id',
    :dependent => :destroy
  accepts_nested_attributes_for :displays

  def ancestors(include_self = false)
    node, nodes = self, []
    nodes << self if include_self
    nodes << node = node.parent while node.parent
    nodes.reverse
  end
  alias_method :parents, :ancestors
  
  def ancestor_ids(include_self = false)
    self.ancestors(include_self).map(&:id)
  end

  def descendants(include_self = false)
    result = (self.children.map{|child| child.descendants} + self.children).flatten
    result << self if include_self
    result
  end

  def descendant_ids(include_self = false)
    self.descendants(include_self).map(&:id)
  end

  # 商品に登録されているカテゴリIDからのカテゴリ担当者のメールアドレスすべてを配列で返却します
  # 商品に登録されているすべてのカテゴリから担当者メールアドレスが取得できなかった場合はさらに親のカテゴリを取得し、
  # 親のカテゴリの担当者メールアドレスを配列にして返します。
  def self.get_staff_emails_for_cc(product)
    email_list = []

    where = Condition.block do |c|
      c.or("id", "=", product.category_1_id) unless product.category_1_id.nil?
      c.or("id", "=", product.category_2_id) unless product.category_2_id.nil?
      c.or("id", "=", product.category_3_id) unless product.category_3_id.nil?
      c.or("id", "=", product.category_4_id) unless product.category_4_id.nil?
      c.or("id", "=", product.category_5_id) unless product.category_5_id.nil?
    end

    unless where.nil?
      product_categories = self.find(:all, :conditions => where)
      product_categories.each do |pc|
        email_list.concat(pc.staff_emails.split(",")) unless pc.staff_emails.nil?
      end

      # メールアドレスが一件も取得できなかった場合、カテゴリの親を検索し、emailが取得できるまで繰り返す
      if email_list.size == 0
        product_categories.each do |pc|
          if (!pc.parent.nil? && !pc.parent.staff_emails.nil?)
            email_list.concat(pc.parent.staff_emails.split(","))
            break
          end
        end
      end
    end
    return email_list
  end

  # 指定されたカテゴリとその祖先を取得します
  def self.get_pankuzu_list(category_code)
    self.find(:first, :conditions=> ["code=?", category_code]).displays_for_topic_path unless category_code.nil?
  end

  # 自身とその祖先のproduct_category_displayを取得します
  def displays_for_topic_path
    pankuzu_list = ProductCategoryDisplay.find(:all, :include => :product_category,
      :conditions => ["locale = ? AND product_category_id in (?)", (I18n.locale || 'ja').to_s, ancestor_ids(true)])
    return nil if pankuzu_list.size == 0
    return pankuzu_list
  end

  #コードの配列を返す
  def self.codes
    result = ProductCategory.find(:all, :select => 'code')
    result.map{|category| category.code }
  end
  
  #すべてのカテゴリを取得して入れ子のの配列でid、コード、表示名を返す（parent_idを指定するとそれより下位のカテゴリを取得します）
  #indentに文字列を指定すると、表示名をindentでインデントしたフラットな配列を返す
  def self.find_all_categories(locale = 'ja', parent_id = nil, indent = nil)
    condition = ["product_category_displays.locale = ? AND product_categories.parent_id IS NULL", locale]
    if parent_id
      condition = ["product_category_displays.locale = ? AND product_categories.parent_id = ?", locale, parent_id]
    end
    categories = find(:all, :include => 'displays', :conditions => condition, :order => 'product_categories.display_order, product_categories.id')
    result = []
    categories.each do |category|
      result << CategoryDisplayAtLocale.new(category)
      if count(:conditions => ['parent_id = ?', category.id]) > 0
        sub_categories = find_all_categories(locale, category.id, indent)
        if indent
          result = result + sub_categories.map{|sc| sc.name = "#{indent}#{sc.name}"; sc }
        else
          result << sub_categories
        end
      end
    end
    return result
  end

  # find_all_categoriesで返すカテゴリ情報を格納するクラス
  class CategoryDisplayAtLocale
    attr_accessor :id, :code, :name
    
    def initialize(category)
      self.id = category.id
      self.code = category.code
      self.name = category.displays.first.name unless category.displays.empty?
    end
  end
end
