# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe CampaignCondition do
  before(:each) do
    @valid_attributes = {
      :campaign_id => 1,
      :no => 1,
      :type_cd => "ty",
      :operator_cd => "op",
      :value => "value for value",
      :connector_cd => "co",
      :brace_begin_count => 1,
      :brace_end_count => 1
    }
  end

  it "should create a new instance given valid attributes" do
    CampaignCondition.create!(@valid_attributes)
  end

  describe "each type_entry" do
    CampaignCondition.type_entries.each do |entry|
      describe "#{entry.id}: #{entry.key.inspect}(#{entry.name})" do
        it "should have :func" do
          entry[:func].should_not be_nil
        end
      end
    end
  end


  describe "build_script" do
    # 不正な条件
    describe "invalid attributes" do
      it "raise_error" do
        cond = CampaignCondition.new
        lambda{ cond.build_script }.should raise_error(ActiveRecord::RecordInvalid)
        cond.errors[:type_cd].should_not be_empty
        cond.errors[:operator_cd].should_not be_empty

        cond.type_key = :user_birthday
        lambda{ cond.build_script }.should raise_error(ActiveRecord::RecordInvalid)
        cond.errors[:type_cd].should be_nil
        cond.errors[:operator_cd].should_not be_empty

        cond.type_cd = nil
        cond.operator_key = :eql
        lambda{ cond.build_script }.should raise_error(ActiveRecord::RecordInvalid)
        cond.errors[:type_cd].should_not be_empty
        cond.errors[:operator_cd].should be_nil
      end

    end
  end

  describe "build_script and match?" do

    # 性別
    describe "for user_gender" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_gender, :operator_key => :eql, :value => 'female')
        cond.build_script.should == "user.sex_key.to_s == 'female'"
        cond.match?(:user => User.new(:sex_key => :male)).should == false
        cond.match?(:user => User.new(:sex_key => :female)).should == true
      end

      it "by not" do
        cond = CampaignCondition.new(:type_key => :user_gender, :operator_key => :not, :value => 'male')
        cond.build_script.should == "user.sex_key.to_s != 'male'"
        cond.match?(:user => User.new(:sex_key => :male)).should == false
        cond.match?(:user => User.new(:sex_key => :female)).should == true
      end
    end

    # 年齢
    describe "for user_age" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_age, :operator_key => :eql, :value => '36')
        cond.build_script.should == "user.age == CampaignCondition.parse_age('36')"
        Date.should_receive(:today).exactly(3).times.and_return(Date.new(2009, 7, 4))
        cond.match?(:user => User.new(:birth_year => 1973, :birth_month => 7, :birth_day => 3)).should == true
        cond.match?(:user => User.new(:birth_year => 1973, :birth_month => 7, :birth_day => 4)).should == true
        cond.match?(:user => User.new(:birth_year => 1973, :birth_month => 7, :birth_day => 5)).should == false

      end

      it "by gte" do
        cond = CampaignCondition.new(:type_key => :user_age, :operator_key => :gte, :value => '65')
        cond.build_script.should == "user.age >= CampaignCondition.parse_age('65')"
        Date.should_receive(:today).exactly(3).times.and_return(Date.new(2010, 3, 31))
        cond.match?(:user => User.new(:birth_year => 1945, :birth_month => 3, :birth_day => 30)).should == true
        cond.match?(:user => User.new(:birth_year => 1945, :birth_month => 3, :birth_day => 31)).should == true
        cond.match?(:user => User.new(:birth_year => 1945, :birth_month => 4, :birth_day => 1)).should == false
      end
    end

    # 誕生日(年月日)
    describe "for user_birthday" do
      it "by gt" do
        cond = CampaignCondition.new(:type_key => :user_birthday, :operator_key => :gt, :value => '1990/4/1')
        cond.build_script.should == "user.birthday > CampaignCondition.parse_date('1990/4/1')"
        cond.match?(:user => User.new(:birth_year => 1990, :birth_month => 3, :birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_year => 1990, :birth_month => 4, :birth_day => 1)).should == false
        cond.match?(:user => User.new(:birth_year => 1990, :birth_month => 4, :birth_day => 2)).should == true
      end
      
      it "by lte" do
        cond = CampaignCondition.new(:type_key => :user_birthday, :operator_key => :lte, :value => '1950/01/01')
        cond.build_script.should == "user.birthday <= CampaignCondition.parse_date('1950/01/01')"
        cond.match?(:user => User.new(:birth_year => 1949, :birth_month => 12, :birth_day => 31)).should == true
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 1)).should == true
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 2)).should == false
      end
      
      it "by eql with today" do
        cond = CampaignCondition.new(:type_key => :user_birthday, :operator_key => :eql, :value => 'today')
        cond.build_script.should == "user.birthday == CampaignCondition.parse_date('today')"
        Date.stub!(:today).and_return(Date.parse('2010/1/1'))
        cond.match?(:user => User.new(:birth_year => 1949, :birth_month => 12, :birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 1)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 2)).should == false
        cond.match?(:user => User.new(:birth_year => 2009, :birth_month => 12, :birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_year => 2010, :birth_month => 1, :birth_day => 1)).should == true
        cond.match?(:user => User.new(:birth_year => 2010, :birth_month => 1, :birth_day => 2)).should == false
      end
    end

    # 誕生日(月日)
    describe "for user_birthday_of_year" do
      it "by eql with today_in_year" do
        cond = CampaignCondition.new(:type_key => :user_birthday_of_year, :operator_key => :eql, :value => '3/4')
        cond.build_script.should == "user.birthday_of_year == CampaignCondition.parse_date_of_year('3/4')"
        cond.match?(:user => User.new(:birth_year => 1949, :birth_month => 3, :birth_day => 3)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 3, :birth_day => 4)).should == true
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 3, :birth_day => 5)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 3, :birth_day => 3)).should == false
        cond.match?(:user => User.new(:birth_year => 1951, :birth_month => 3, :birth_day => 4)).should == true
        cond.match?(:user => User.new(:birth_year => 1951, :birth_month => 3, :birth_day => 5)).should == false
      end

      it "by eql with today_in_year" do
        cond = CampaignCondition.new(:type_key => :user_birthday_of_year, :operator_key => :eql, :value => 'today')
        cond.build_script.should == "user.birthday_of_year == CampaignCondition.parse_date_of_year('today')"
        Date.stub!(:today).and_return(Date.parse('2010/1/1'))
        cond.match?(:user => User.new(:birth_year => 1949, :birth_month => 12, :birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 1)).should == true
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 1, :birth_day => 2)).should == false
        cond.match?(:user => User.new(:birth_year => 1950, :birth_month => 12, :birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_year => 1951, :birth_month => 1, :birth_day => 1)).should == true
        cond.match?(:user => User.new(:birth_year => 1951, :birth_month => 1, :birth_day => 2)).should == false
      end

    end

    # 誕生日(日)
    describe "for user_birth_day" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_birth_day, :operator_key => :eql, :value => '1')
        cond.build_script.should == "user.birth_day.to_i == CampaignCondition.parse_day_of_month('1')"
        cond.match?(:user => User.new(:birth_day => 31)).should == false
        cond.match?(:user => User.new(:birth_day => 1)).should == true
        cond.match?(:user => User.new(:birth_day => 2)).should == false
      end

      it "by eql with today" do
        cond = CampaignCondition.new(:type_key => :user_birth_day, :operator_key => :eql, :value => 'today')
        cond.build_script.should == "user.birth_day.to_i == CampaignCondition.parse_day_of_month('today')"
        Date.stub!(:today).and_return(Date.parse('2010/4/2'))
        cond.match?(:user => User.new(:birth_day => 1)).should == false
        cond.match?(:user => User.new(:birth_day => 2)).should == true
        cond.match?(:user => User.new(:birth_day => 3)).should == false
      end
    end

    # 誕生日(月)
    describe "for user_birth_month" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_birth_month, :operator_key => :eql, :value => '4')
        cond.build_script.should == "user.birth_month.to_i == CampaignCondition.parse_month('4')"
        cond.match?(:user => User.new(:birth_month => 3)).should == false
        cond.match?(:user => User.new(:birth_month => 4)).should == true
        cond.match?(:user => User.new(:birth_month => 5)).should == false
      end

      it "by eql with today" do
        cond = CampaignCondition.new(:type_key => :user_birth_month, :operator_key => :eql, :value => 'today')
        cond.build_script.should == "user.birth_month.to_i == CampaignCondition.parse_month('today')"
        Date.stub!(:today).and_return(Date.parse('2010/4/2'))
        cond.match?(:user => User.new(:birth_month => 3)).should == false
        cond.match?(:user => User.new(:birth_month => 4)).should == true
        cond.match?(:user => User.new(:birth_month => 5)).should == false
      end
    end

    # 誕生日(年)
    describe "for user_birth_year" do
      it "by gte" do
        cond = CampaignCondition.new(:type_key => :user_birth_year, :operator_key => :gte, :value => '1970')
        cond.build_script.should == "user.birth_year.to_i >= CampaignCondition.parse_year('1970')"
        cond.match?(:user => User.new(:birth_year => 1969)).should == false
        cond.match?(:user => User.new(:birth_year => 1970)).should == true
        cond.match?(:user => User.new(:birth_year => 1971)).should == true
      end

      it "by lte with today" do
        cond = CampaignCondition.new(:type_key => :user_birth_year, :operator_key => :lte, :value => 'today')
        cond.build_script.should == "user.birth_year.to_i <= CampaignCondition.parse_year('today')"
        Date.stub!(:today).and_return(Date.parse('2010/4/2'))
        cond.match?(:user => User.new(:birth_year => 2009)).should == true
        cond.match?(:user => User.new(:birth_year => 2010)).should == true
        cond.match?(:user => User.new(:birth_year => 2011)).should == false
      end
    end

    describe "now" do
      before(:each) do
        @now = Time.parse('2009/10/1')
        Time.stub!(:now).and_return(@now)
        Date.stub!(:today).and_return(@now.to_date)
      end

      # 現時刻(年月日)
      describe "for now_date" do
        it "by gte" do
          cond = CampaignCondition.new(:type_key => :now_date, :operator_key => :gte, :value => '2009/10/01')
          cond.build_script.should == "Date.today >= CampaignCondition.parse_date('2009/10/01')"
          cond.match?.should == true
        end

        it "by gt" do
          cond = CampaignCondition.new(:type_key => :now_date, :operator_key => :gt, :value => '2009/10/01')
          cond.build_script.should == "Date.today > CampaignCondition.parse_date('2009/10/01')"
          cond.match?.should == false
        end

        it "by lte" do
          cond = CampaignCondition.new(:type_key => :now_date, :operator_key => :lte, :value => '2009/10/01')
          cond.build_script.should == "Date.today <= CampaignCondition.parse_date('2009/10/01')"
          cond.match?.should == true
        end

        it "by lt" do
          cond = CampaignCondition.new(:type_key => :now_date, :operator_key => :lt, :value => '2009/10/01')
          cond.build_script.should == "Date.today < CampaignCondition.parse_date('2009/10/01')"
          cond.match?.should == false
        end
      end

      # 現時刻(月日)
      describe "for now_date_of_year" do
        it "by eql true" do
          cond = CampaignCondition.new(:type_key => :now_date_of_year, :operator_key => :eql, :value => '10/01')
          cond.build_script.should == "Date.today.strftime('%m/%d') == CampaignCondition.parse_date_of_year('10/01')"
          cond.match?.should == true
        end

        it "by eql false" do
          cond = CampaignCondition.new(:type_key => :now_date_of_year, :operator_key => :eql, :value => '10/02')
          cond.build_script.should == "Date.today.strftime('%m/%d') == CampaignCondition.parse_date_of_year('10/02')"
          cond.match?.should == false
        end
      end

      # 現時刻(年)
      describe "for now_year" do
        it "by eql true" do
          cond = CampaignCondition.new(:type_key => :now_year, :operator_key => :eql, :value => '2009')
          cond.build_script.should == "Date.today.year.to_i == CampaignCondition.parse_year('2009')"
          cond.match?.should == true
        end

        it "by eql false" do
          cond = CampaignCondition.new(:type_key => :now_year, :operator_key => :eql, :value => '2010')
          cond.build_script.should == "Date.today.year.to_i == CampaignCondition.parse_year('2010')"
          cond.match?.should == false
        end
      end

      # 現時刻(月)
      describe "for now_month" do
        it "by eql true" do
          cond = CampaignCondition.new(:type_key => :now_month, :operator_key => :eql, :value => '10')
          cond.build_script.should == "Date.today.month.to_i == CampaignCondition.parse_month('10')"
          cond.match?.should == true
        end

        it "by eql false" do
          cond = CampaignCondition.new(:type_key => :now_month, :operator_key => :eql, :value => '9')
          cond.build_script.should == "Date.today.month.to_i == CampaignCondition.parse_month('9')"
          cond.match?.should == false
        end
      end

      # 現時刻(日)
      describe "for now_day" do
        it "by eql true" do
          cond = CampaignCondition.new(:type_key => :now_day, :operator_key => :eql, :value => '1')
          cond.build_script.should == "Date.today.day.to_i == CampaignCondition.parse_day_of_month('1')"
          cond.match?.should == true
        end

        it "by eql false" do
          cond = CampaignCondition.new(:type_key => :now_day, :operator_key => :eql, :value => '2')
          cond.build_script.should == "Date.today.day.to_i == CampaignCondition.parse_day_of_month('2')"
          cond.match?.should == false
        end
      end

      # 現時刻(曜日)
      describe "for now_day_of_week" do
        # >> Date.new(2009, 10, 1).cwday
        # => 4

        it "by eql true" do
          cond = CampaignCondition.new(:type_key => :now_day_of_week, :operator_key => :eql, :value => '木')
          cond.build_script.should == "Date.today.cwday.to_i == CampaignCondition.parse_day_of_week('木')"
          cond.match?.should == true
        end

        it "by eql true Eng" do
          cond = CampaignCondition.new(:type_key => :now_day_of_week, :operator_key => :eql, :value => 'Thu')
          cond.build_script.should == "Date.today.cwday.to_i == CampaignCondition.parse_day_of_week('Thu')"
          cond.match?.should == true
        end

        it "by eql false" do
          cond = CampaignCondition.new(:type_key => :now_day_of_week, :operator_key => :eql, :value => '水')
          cond.build_script.should == "Date.today.cwday.to_i == CampaignCondition.parse_day_of_week('水')"
          cond.match?.should == false
        end

        it "by eql false Eng" do
          cond = CampaignCondition.new(:type_key => :now_day_of_week, :operator_key => :eql, :value => 'Wed')
          cond.build_script.should == "Date.today.cwday.to_i == CampaignCondition.parse_day_of_week('Wed')"
          cond.match?.should == false
        end
      end
    end

    # 国
    describe "for user_country" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_country, :operator_key => :eql, :value => 'JP')
        cond.build_script.should == "user.address_country_code == 'JP'"
        user = User.new
        user.user_address = UserAddress.new
        user.user_address.country_code = 'GB'; cond.match?(:user => user).should == false
        user.user_address.country_code = 'US'; cond.match?(:user => user).should == false
        user.user_address.country_code = 'JP'; cond.match?(:user => user).should == true
      end
    end

    # 住所
    describe "for user_address" do
      it "by eql" do
        cond = CampaignCondition.new(:type_key => :user_address, :operator_key => :eql, :value => '愛知県名古屋市')
        cond.build_script.should == "user.address_for_campaign_condition == '愛知県名古屋市'"
        user = User.new
        user.user_address = UserAddress.new(:locale => 'ja', :country_code => 'JP')
        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '名古屋市'
        user.user_address.address_3 = nil
        user.address_for_campaign_condition.should == '愛知県名古屋市'
        cond.match?(:user => user).should == true

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '名古屋市中区栄１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == false

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '豊田市豊田１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == false

        user.user_address.address_1 = '岐阜県'
        user.user_address.address_2 = '豊田市豊田１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == false
      end

      it "by not" do
        cond = CampaignCondition.new(:type_key => :user_address, :operator_key => :not, :value => '愛知県')
        cond.build_script.should == "user.address_for_campaign_condition != '愛知県'"
        user = User.new
        user.user_address = UserAddress.new(:locale => 'ja', :country_code => 'JP')
        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = nil
        user.user_address.address_3 = nil
        cond.match?(:user => user).should == false

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '名古屋市中区栄１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == true

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '豊田市豊田１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == true

        user.user_address.address_1 = '岐阜県'
        user.user_address.address_2 = '豊田市豊田１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == true
      end

      it "by like" do
        cond = CampaignCondition.new(:type_key => :user_address, :operator_key => :like, :value => '愛知県名古屋市')
        cond.build_script.should == "user.address_for_campaign_condition =~ Regexp.new(Regexp.escape('愛知県名古屋市'))"
        user = User.new
        user.user_address = UserAddress.new(:locale => 'ja', :country_code => 'JP')
        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = nil
        user.user_address.address_3 = nil
        cond.match?(:user => user).should == false

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '名古屋市'
        user.user_address.address_3 = nil
        cond.match?(:user => user).should == true

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '名古屋市中区栄１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == true

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '豊田市豊田１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == false

        user.user_address.address_1 = '岐阜県'
        user.user_address.address_2 = '名古屋市中区栄１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == false
      end

      it "by not_like" do
        cond = CampaignCondition.new(:type_key => :user_address, :operator_key => :not_like, :value => '愛知県名古屋市')
        cond.build_script.should == "user.address_for_campaign_condition !~ Regexp.new(Regexp.escape('愛知県名古屋市'))"
        user = User.new
        user.user_address = UserAddress.new(:locale => 'ja', :country_code => 'JP')
        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = nil
        user.user_address.address_3 = nil
        cond.match?(:user => user).should == true

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '名古屋市'
        user.user_address.address_3 = nil
        cond.match?(:user => user).should == false

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '名古屋市中区栄１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == false

        user.user_address.address_1 = '愛知県'
        user.user_address.address_2 = '豊田市豊田１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == true

        user.user_address.address_1 = '岐阜県'
        user.user_address.address_2 = '名古屋市中区栄１−２−３'
        user.user_address.address_3 = ''
        cond.match?(:user => user).should == true
      end
    end

    # 職業 
    # TODO 確認: これはどこで入力されている？
    describe "for user_occupation" do
    end

    # 会員ランク
    # TODO when 会員ランク実装 
    describe "for user_rank" do
    end

    # 保有ポイント
    # TODO ポイント実装
    describe "for user_total_point" do
    end

    # 小計額 TODO いい名前！
    # キャンペーンによる効果を与える前の合計金額。
    # 「購入金額」は顧客が支払う金額なのでキャンペーンによって変わり得ます。
    describe "for cart_sum_product_price" do
    end

    describe "会員種別" do
      it "" do
        cond = CampaignCondition.new(:type_key => :member_category_div, :operator_key => :eql, :value => '001')
        cond.build_script.should == "user.member_category_div == '001'"
        user = User.new(:member_category_div => '1')
        user.user_address = UserAddress.new(:locale => 'ja', :country_code => 'JP')
        user.user_address.address_1 = '愛知県'
        cond.match?(:user => user).should == false

        user.member_category_div = '001'
        cond.match?(:user => user).should == true
      end
    end
  end


  describe "build with brace and connector" do
    it "shouldn't have connector if first condition" do
      cond = CampaignCondition.new
      cond.no = 0
      cond.should_receive(:build_script).and_return("script")
      cond.build.should == "script"
    end

    it "shouldn't have connector if first condition" do
      cond = CampaignCondition.new
      cond.no = 0
      cond.brace_begin_count = 2
      cond.brace_end_count = 0
      cond.should_receive(:build_script).and_return("script")
      cond.build.should == "((script"
    end

    it "should have connector after second condition" do
      cond = CampaignCondition.new
      cond.no = 1
      cond.connector_key = :and
      cond.brace_begin_count = 0
      cond.brace_end_count = 0
      cond.should_receive(:build_script).and_return("script")
      cond.build.should == "&& script"
    end

    it "should have connector after second condition" do
      cond = CampaignCondition.new
      cond.no = 2
      cond.connector_key = :or
      cond.brace_begin_count = 0
      cond.brace_end_count = 2
      cond.should_receive(:build_script).and_return("script")
      cond.build.should == "|| script))"
    end
  end


  describe "parsing date" do
    describe "parse_date" do
      it "with valid" do
        CampaignCondition.parse_date("2010/4/1").should == Date.parse("2010/4/1")
      end

      it "with today" do
        Date.stub!(:today).and_return(Date.parse("2010/4/30"))
        CampaignCondition.parse_date("today").should == Date.parse("2010/4/30")
      end

      it "with invalid" do
        lambda{ CampaignCondition.parse_date("2010/4/31") }.should raise_error(ArgumentError)
      end
    end

    describe "parse_date_of_year" do
      it "with valid" do
        CampaignCondition.parse_date_of_year("4/1").should == "04/01"
        CampaignCondition.parse_date_of_year("04/1").should == "04/01"
        CampaignCondition.parse_date_of_year("04/01").should == "04/01"
        CampaignCondition.parse_date_of_year("2009/04/01").should == "04/01"
      end

      it "with today" do
        Date.stub!(:today).and_return(Date.parse("2010/4/30"))
        CampaignCondition.parse_date_of_year("today").should == "04/30"
      end

      it "with invalid" do
        lambda{ CampaignCondition.parse_date_of_year("13/31") }.should raise_error(ArgumentError)
        lambda{ CampaignCondition.parse_date_of_year("1/32") }.should raise_error(ArgumentError)
        lambda{ CampaignCondition.parse_date_of_year("2009/02/31") }.should raise_error(ArgumentError)
        lambda{ CampaignCondition.parse_date_of_year("2010/4/31") }.should raise_error(ArgumentError)
      end
    end

    describe "parse_day_of_month" do
      it "with valid" do
        CampaignCondition.parse_day_of_month("1").should == 1
        CampaignCondition.parse_day_of_month("31").should == 31
        CampaignCondition.parse_day_of_month("32").should == 32
        CampaignCondition.parse_day_of_month('100').should == 100
      end

      it "with today" do
        Date.stub!(:today).and_return(Date.parse("2010/4/30"))
        CampaignCondition.parse_day_of_month("today").should == 30
      end
    end

    describe "parse_month" do
      it "with valid" do
        CampaignCondition.parse_month("1").should == 1
        CampaignCondition.parse_month("12").should == 12
        CampaignCondition.parse_month("13").should == 13
        CampaignCondition.parse_month('100').should == 100
      end

      it "with today" do
        Date.stub!(:today).and_return(Date.parse("2010/4/30"))
        CampaignCondition.parse_month("today").should == 4
      end
    end

    describe "parse_year" do
      it "with valid" do
        CampaignCondition.parse_year("1950").should == 1950
        CampaignCondition.parse_year("1975").should == 1975
        CampaignCondition.parse_year("2000").should == 2000
        CampaignCondition.parse_year('2300').should == 2300
      end

      it "with today" do
        Date.stub!(:today).and_return(Date.parse("2010/4/30"))
        CampaignCondition.parse_year("today").should == 2010
      end
    end

    describe "parse_day_of_week" do
      it "with valid" do
        CampaignCondition.parse_day_of_week('月').should == 1
        CampaignCondition.parse_day_of_week('火').should == 2
        CampaignCondition.parse_day_of_week('水').should == 3
        CampaignCondition.parse_day_of_week('木').should == 4
        CampaignCondition.parse_day_of_week('金').should == 5
        CampaignCondition.parse_day_of_week('土').should == 6
        CampaignCondition.parse_day_of_week('日').should == 7
        
        CampaignCondition.parse_day_of_week('月曜日').should == 1
        CampaignCondition.parse_day_of_week('火曜日').should == 2
        CampaignCondition.parse_day_of_week('水曜日').should == 3
        CampaignCondition.parse_day_of_week('木曜日').should == 4
        CampaignCondition.parse_day_of_week('金曜日').should == 5
        CampaignCondition.parse_day_of_week('土曜日').should == 6
        CampaignCondition.parse_day_of_week('日曜日').should == 7

        CampaignCondition.parse_day_of_week('Mon').should == 1
        CampaignCondition.parse_day_of_week('Tue').should == 2
        CampaignCondition.parse_day_of_week('Wed').should == 3
        CampaignCondition.parse_day_of_week('Thu').should == 4
        CampaignCondition.parse_day_of_week('Fri').should == 5
        CampaignCondition.parse_day_of_week('Sat').should == 6
        CampaignCondition.parse_day_of_week('Sun').should == 7

        CampaignCondition.parse_day_of_week('Monday').should == 1
        CampaignCondition.parse_day_of_week('Tuesday').should == 2
        CampaignCondition.parse_day_of_week('Wednesday').should == 3
        CampaignCondition.parse_day_of_week('Thursday').should == 4
        CampaignCondition.parse_day_of_week('Friday').should == 5
        CampaignCondition.parse_day_of_week('Saturday').should == 6
        CampaignCondition.parse_day_of_week('Sunday').should == 7
      end

      it "with invalid" do
        CampaignCondition.parse_day_of_week(nil).should == nil
        CampaignCondition.parse_day_of_week('').should == nil
        CampaignCondition.parse_day_of_week('UNKNOWN').should == nil
      end
    end

    describe "parse_time_of_day" do
      it "with valid" do
        CampaignCondition.parse_time_of_day('1:3').should == "01:03"
        CampaignCondition.parse_time_of_day('1:30').should == "01:30"
        CampaignCondition.parse_time_of_day('01:30').should == "01:30"
        CampaignCondition.parse_time_of_day('20:30').should == "20:30"
        CampaignCondition.parse_time_of_day('20:30:40').should == "20:30"

        CampaignCondition.parse_time_of_day('2007/12/31 20:30').should == "20:30"
        CampaignCondition.parse_time_of_day('12/31 1:0').should == "01:00"
      end

      it "with invalid" do
        lambda{ CampaignCondition.parse_time_of_day('12/20')}.should raise_error(ArgumentError)
        lambda{ CampaignCondition.parse_time_of_day('25:06')}.should raise_error(ArgumentError)
        lambda{ CampaignCondition.parse_time_of_day('23:66')}.should raise_error(ArgumentError)
      end
    end

  end


  describe "parse_age" do
    it "with valid" do
      CampaignCondition.parse_age('14').should == 14
      CampaignCondition.parse_age('18歳').should == 18
      CampaignCondition.parse_age('29才').should == 29
      CampaignCondition.parse_age('39').should == 39
    end
    
    it "with invalid" do
      lambda{ CampaignCondition.parse_age(nil) }.should raise_error(ArgumentError)
      lambda{ CampaignCondition.parse_age('') }.should raise_error(ArgumentError)
      lambda{ CampaignCondition.parse_age('ABC') }.should raise_error(ArgumentError)
    end
  end


  describe "create" do
    it "valid" do
      cond = CampaignCondition.new(:type_key => :user_gender, :operator_key => :eql, :value => 'male')
      cond.campaign_id = 1 # これはモデルが自動で設定できません 
      cond.no = 0 # これはCampaiognが自動で設定してくれます
      cond.save
      cond.errors.full_messages.should == []
      cond.connector_key.should == :and
      cond.brace_begin_count.should == 0
      cond.brace_end_count.should == 0
    end
  end

  describe "operator_cd_values" do
    it "operator_cd_values" do
      result = CampaignCondition.operator_cd_values("22")
      result.should == [["等しい","05"]]
      result = CampaignCondition.operator_cd_values("21")
      result.should == [["より小さい", "01"], ["より大きい", "02"], ["以下", "03"], ["以上", "04"],
        ["等しい", "05"], ["以外", "06"], ["を含まない", "08"]]
    end
  end
  describe "value_for_type" do
    it "value_for_type" do
      cam = CampaignCondition.new(:type_cd => '22', :value => "001")
      cam.value_for_type.should == "ParentsChina会員"
      cam = CampaignCondition.new(:type_cd => '01', :value => "001")
      cam.value_for_type.should == "001"
    end
  end
end
